<?php

namespace Pg\Modules\Countries\Models;

use \Pg\Libraries\ModuleConfigTrait;

/**
 * Countries module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

define('COUNTRIES_TABLE', DB_PREFIX . 'cnt_countries');
define('REGIONS_TABLE', DB_PREFIX . 'cnt_regions');
define('CITIES_TABLE', DB_PREFIX . 'cnt_cities');
define('DISTRICTS_TABLE', DB_PREFIX . 'cnt_districts');
define('CACHE_COUNTRIES_TABLE', DB_PREFIX . 'cnt_cache_countries');
define('CACHE_REGIONS_TABLE', DB_PREFIX . 'cnt_cache_regions');
define('GEOBASE_URL', 'http://download.pilotgroup.net/geobase/wget/');

/**
 * Countries main model
 *
 * @package PG_RealEstate
 * @subpackage Countries
 *
 * @category    models
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Countries_model extends \Model
{

    use ModuleConfigTrait;

    const MODULE_GID = 'countries';

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Insert queries per request
     *
     * @var integer
     */
    protected $db_insert_step = 100;

    /**
     * Cachable country attributes
     *
     * @var array
     */
    protected $cache_attrs_country = array(
        'id',
        'code',
        'languages',
        'name',
        'areainsqkm',
        'continent',
        'currency',
        'region_update_date',
    );

    /**
     * Cachable region attributes
     *
     * @var array
     */
    protected $cache_attrs_region = array(
        'id',
        'country_code',
        'code',
        'id_region',
        'name',
    );

    /**
     * Properties of country object in data source
     *
     * @var array
     */
    protected $attrs_country = array(
        'id',
        'code',
        'languages',
        'name',
        'areainsqkm',
        'continent',
        'currency',
        'priority',
        'sorted',
    );

    /**
     * Properties of region object in data source
     *
     * @var array
     */
    protected $attrs_region = array(
        'id',
        'country_code',
        'code',
        'name',
        'priority',
        'sorted',
    );

    /**
     * Properties of city object in data source
     *
     * @var array
     */
    protected $attrs_city = array(
        'id',
        'id_region',
        'name',
        'latitude',
        'longitude',
        'country_code',
        'region_code',
        'priority',
        'sorted',
    );

    /**
     * Properties of district object in data source
     *
     * @var array
     */
    protected $attrs_district = array(
        'id',
        'id_city',
        'id_region',
        'name',
        'latitude',
        'longitude',
        'country_code',
        'region_code',
        'priority',
        'sorted',
    );

    /**
     * Use install city from file
     *
     * @var boolean
     */
    protected $use_infile_city_install = true;

    /**
     * Install cities per request
     *
     * @var integer
     */
    protected $city_install_step = 100;

    /**
     * Temporal city identifier on server
     *
     * @var integer
     */
    protected $temp_server_city_id = 0;

    /**
     * Countries cache
     *
     * @var array
     */
    protected $country_cache = array();

    /**
     * Regions cache
     *
     * @var array
     */
    protected $region_cache = array();

    /**
     * Cities cache
     *
     * @var array
     */
    protected $city_cache = array();

    /**
     * Districts cache
     *
     * @var array
     */
    protected $district_cache = array();

    /**
     * Constructor
     *
     * @return Countries_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->use_infile_city_install = $this->ci->pg_module->get_module_config('countries', 'use_infile_city_install');
    }

    private function getModuleConfigList()
    {
        return [
            'use_district' => [
                'key' => 'use_district',
                'type' => 'checkbox',
                'name' => null,
                'value' => null,
            ],
            'output_country_format' => [
                'key' => 'output_country_format',
                'type' => 'text',
                'name' => null,
                'value' => null,
            ],
            'output_region_format' => [
                'key' => 'output_region_format',
                'type' => 'text',
                'name' => null,
                'value' => null,
            ],
            'output_city_format' => [
                'key' => 'output_city_format',
                'type' => 'text',
                'name' => null,
                'value' => null,
            ],
            'output_district_format' => [
                'key' => 'output_district_format',
                'type' => 'text',
                'name' => null,
                'value' => null,
            ],
            'output_address_format' => [
                'key' => 'output_address_format',
                'type' => 'text',
                'name' => null,
                'value' => null,
            ],
        ];
    }

    /**
     *  Upload country list from server
     *
     *  Put it in base cache. Return countries list.
     *
     * @return array
     */
    private function wget_countries()
    {
        $languages = array_values($this->ci->pg_language->languages);

        $langs_code = array();
        foreach ($languages as $language) {
            $langs_code[] = $language['code'];
        }

        $this->ci->load->library('Snoopy');
        $res = $this->ci->snoopy->fetch(GEOBASE_URL . 'get_countries/' . implode('-', $langs_code) . '/');

        if (!$res || !preg_match('/200 OK/i', $this->ci->snoopy->headers[0])) {
            return false;
        }
        $temp_geo_data = $this->ci->snoopy->results;

        $data = array();
        $temp_geo_array = preg_split('/\n/', $temp_geo_data);
        foreach ($temp_geo_array as $geo) {
            if (!strlen(trim($geo))) {
                continue;
            }
            $geo_array = preg_split("/\t/", $geo);

            $langs_names = array();
            foreach ($languages as $i => $language) {
                $langs_names['lang_' . $language['id']] = isset($geo_array[$i + 19]) ? $geo_array[$i + 19] : $geo_array[4];
            }

            $data[] = array_merge(array(
                'code' => $geo_array[0],
                'languages' => $geo_array[0],
                'name' => $geo_array[4],
                'areainsqkm' => $geo_array[6],
                'continent' => $geo_array[8],
                'currency' => $geo_array[10],
                'region_update_date' => '0000-00-00 00:00:00',
                    ), $langs_names);
        }

        return $data;
    }

    /**
     * Upload region list from server
     *
     * Put it in base cache. Return region list.
     *
     * @param string $country_code country code
     *
     * @return array
     */
    private function wget_regions($country_code)
    {
        $languages = array_values($this->ci->pg_language->languages);

        $langs_code = array();
        foreach ($languages as $language) {
            $langs_code[] = $language['code'];
        }

        $this->ci->load->library('Snoopy');
        $res = $this->ci->snoopy->fetch(GEOBASE_URL . 'get_regions/' . $country_code . '/' . implode('-', $langs_code) . '/');

        if (!$res || !preg_match('/200 OK/i', $this->ci->snoopy->headers[0])) {
            return false;
        }
        $temp_geo_data = $this->ci->snoopy->results;

        $data = array();
        $temp_geo_array = preg_split('/\n/', $temp_geo_data);
        foreach ($temp_geo_array as $geo) {
            if (!strlen(trim($geo))) {
                continue;
            }

            $geo_array = preg_split("/\t/", $geo);

            $langs_names = array();
            foreach ($languages as $i => $language) {
                $langs_names['lang_' . $language['id']] = isset($geo_array[$i + 4]) ? $geo_array[$i + 4] : $geo_array[3];
            }

            $data[] = array_merge(array(
                'country_code' => $geo_array[1],
                'code' => $geo_array[2],
                'id_region' => $geo_array[0],
                'name' => $geo_array[3],
                    ), $langs_names);
        }

        return $data;
    }

    /**
     * Upload city list from server
     *
     * Put it in base cache. Return city list.
     *
     * @param string  $country_code     country code
     * @param integer $region_server_id region identifier
     * @param array   $region_data      region data
     *
     * @return array
     */
    private function wget_cities($country_code, $region_server_id, $region_data)
    {
        //// get cities from server and return cities list
        //// if returned data && infile == true - save file and return filepath
        //// if returned data && infile == false - return data array
        //// if returned install clear and return

        $languages = $this->ci->pg_language->languages;
        ksort($languages);
        $languages = array_values($languages);

        $langs_code = array();
        foreach ($languages as $language) {
            $langs_code[] = $language['code'];
        }

        $this->ci->load->library('Snoopy');
        $res = $this->ci->snoopy->fetch(GEOBASE_URL . 'get_regions_cities/' . $country_code . "/" . $region_server_id . "/" . $this->temp_server_city_id . '/' . implode('-',
                                                                                                                                                                         $langs_code) . '/');

        if (!$res || !preg_match('/200 OK/i', $this->ci->snoopy->headers[0])) {
            return false;
        }

        $temp_geo_data = $this->ci->snoopy->results;

        if ('installed' === trim($temp_geo_data)) {
            $this->temp_server_city_id = 0;

            return true;
        }

        $data = array();
        $temp_geo_array = preg_split('/\n/', $temp_geo_data);
        foreach ($temp_geo_array as $geo) {
            if (!strlen(trim($geo))) {
                continue;
            }
            $geo_array = preg_split("/\t/", $geo);

            $langs_names = array();
            foreach ($languages as $i => $language) {
                $langs_names['lang_' . $language['id']] = isset($geo_array[$i + 20]) ? $geo_array[$i + 20] : $geo_array[2];
            }
            $data[] = array_merge(array(
                'id' => null,
                'id_region' => $region_data["id"],
                'name' => $geo_array[2],
                'latitude' => $geo_array[5],
                'longitude' => $geo_array[6],
                'country_code' => $country_code,
                'region_code' => $region_data["code"],
                'priority' => 0,
                'sorted' => 0,
                    ), $langs_names);
            $this->temp_server_city_id = $geo_array[0];
        }

        $return = array("data" => $data, "file" => '');
        if ($this->use_infile_city_install) {
            $infile = "";
            foreach ($data as $city) {
                $infile .= implode("\t", $city) . "\n";
            }
            $path_to_file = TEMPPATH . 'countries/regions_' . $country_code . '.txt';
            $this->ci->load->helper('file');
            if (!write_file($path_to_file, $infile)) {
                $this->use_infile_city_install = false;

                return $return;
            } else {
                @chmod($path_to_file, 0777);
                $return["file"] = $path_to_file;

                return $return;
            }
        } else {
            return $return;
        }
    }

    /**
     * Return country list from cache
     *
     * If cache is empty or expiries - wget_countries
     * Save update date in module settings
     *
     * @return array
     */
    public function get_cache_countries()
    {
        $this->wget_countries();
        $expiried_period = $this->ci->pg_module->get_module_config('countries', 'countries_update_period');

        $lang_id = $this->ci->pg_language->current_lang_id;

        $last_update = $this->ci->pg_module->get_module_config('countries', 'countries_last_update');

        $langs_names = array();
        foreach ($this->pg_language->languages as $i => $language) {
            $langs_names[] = 'lang_' . $language['id'];
        }

        $this->ci->db->select(implode(", ", array_merge($this->cache_attrs_country, $langs_names)))->from(CACHE_COUNTRIES_TABLE)->order_by('lang_' . $lang_id . ' ASC')->order_by('name ASC');
        $results = $this->ci->db->get()->result_array();

        if (empty($results) || (!empty($last_update) && $last_update + $expiried_period < time()) || empty($last_update)) {
            $results = $this->wget_countries();
            if (empty($results)) {
                return array();
            }

            $counter = 0;
            $data_count = count($results);
            $this->ci->db->query('TRUNCATE TABLE ' . CACHE_COUNTRIES_TABLE . '');

            $start_sql = "INSERT INTO " . CACHE_COUNTRIES_TABLE . " (" . implode(',',
                                                                                 array_merge(array('code', 'languages', 'name', 'areainsqkm', 'continent', 'currency', 'region_update_date'),
                                                                                             $langs_names)) . ") VALUES  ";

            while ($counter < $data_count) {
                unset($strings);
                $temp_geo = array_slice($results, $counter, $this->db_insert_step);
                foreach ($temp_geo as $data) {
                    $lang_string = '';
                    foreach ($langs_names as $lang_name) {
                        $lang_string .= ", '" . addslashes($data[$lang_name]) . "'";
                    }
                    $strings[] = "( '" . $data["code"] . "', '" . $data["languages"] . "', '" . addslashes($data["name"]) . "', '" . $data["areainsqkm"] . "', '" . $data["continent"] . "', '" . $data["currency"] . "', '" . $data["region_update_date"] . "'" . $lang_string . ")";
                }

                $query = $start_sql . implode(", ", $strings);
                $this->ci->db->query($query);
                $counter = $counter + $this->db_insert_step;
            }

            $this->ci->pg_module->set_module_config('countries', 'countries_last_update', time());
        }

        foreach ($results as $key => $result) {
            if (empty($result['lang_' . $lang_id])) {
                continue;
            }
            $results[$key]['name'] = $result['lang_' . $lang_id];
        }

        return $results;
    }

    /**
     * Return country object from cache by code
     *
     * @param string $country_code country code
     *
     * @return array
     */
    public function get_cache_country_by_code($country_code)
    {
        $langs_names = array();
        foreach ($this->pg_language->languages as $i => $language) {
            $langs_names[] = 'lang_' . $language['id'];
        }
        $this->ci->db->select(implode(", ", array_merge($this->cache_attrs_country, $langs_names)))->from(CACHE_COUNTRIES_TABLE)->where("code",
                                                                                                                                    $country_code);
        $data = $this->ci->db->get()->result_array();
        if (!empty($data)) {
            $lang_id = $this->pg_language->current_lang_id;
            if (!empty($data[0]['lang_' . $lang_id])) {
                $data[0]['name'] = $data[0]['lang_' . $lang_id];
            }

            return $data[0];
        }

        return array();
    }

    /**
     * Return region list from cache
     *
     * If cache is empty or expiries - wget_regions
     * Save update date in cache region table
     *
     * @param string $country_code       country code
     * @param array  $country_cache_data country data from cache
     *
     * @return array
     */
    public function get_cache_regions($country_code, $country_cache_data = array())
    {
        $expiried_period = $this->ci->pg_module->get_module_config('countries', 'countries_update_period');

        if (empty($country_cache_data)) {
            $country_cache_data = $this->get_cache_country_by_code($country_code);
        }

        $lang_id = $this->ci->pg_language->current_lang_id;

        $last_update = (!empty($country_cache_data["region_update_date"])) ? $country_cache_data["region_update_date"] : "0000-00-00 00:00:00";
        $last_update = intval(strtotime($last_update));

        $langs_names = array();
        foreach ($this->pg_language->languages as $i => $language) {
            $langs_names[] = 'lang_' . $language['id'];
        }

        $this->ci->db->select(implode(", ", array_merge($this->cache_attrs_region, $langs_names)))->from(CACHE_REGIONS_TABLE)->where('country_code',
                                                                                                                                 $country_code)->order_by('lang_' . $lang_id . ' ASC')->order_by('code ASC');
        $results = $this->ci->db->get()->result_array();

        if (empty($results) || (!empty($last_update) && $last_update + $expiried_period < time()) || empty($last_update)) {
            $results = $this->wget_regions($country_code);
            if (empty($results)) {
                return array();
            }

            $counter = 0;
            $data_count = count($results);
            $this->ci->db->where('country_code', $country_code);
            $this->ci->db->delete(CACHE_REGIONS_TABLE);

            $start_sql = "INSERT INTO " . CACHE_REGIONS_TABLE . " (" . implode(',',
                                                                               array_merge(array('country_code', 'code', 'id_region', 'name'),
                                                                                           $langs_names)) . ") VALUES  ";

            while ($counter < $data_count) {
                unset($strings);
                $temp_geo = array_slice($results, $counter, $this->db_insert_step);
                foreach ($temp_geo as $data) {
                    $lang_string = '';
                    foreach ($langs_names as $lang_name) {
                        $lang_string .= ", '" . addslashes($data[$lang_name]) . "'";
                    }
                    $strings[] = "('" . $data["country_code"] . "', '" . $data["code"] . "', '" . $data["id_region"] . "', '" . addslashes($data["name"]) . "'" . $lang_string . ")";
                }

                $query = $start_sql . implode(", ", $strings);
                $this->ci->db->query($query);
                $counter = $counter + $this->db_insert_step;
            }

            $cdata["region_update_date"] = date('Y-m-d H:i:s');
            $this->ci->db->where("code", $country_code);
            $this->ci->db->update(CACHE_COUNTRIES_TABLE, $cdata);
        }

        foreach ($results as $key => $result) {
            if (empty($result['lang_' . $lang_id])) {
                continue;
            }
            $results[$key]['name'] = $result['lang_' . $lang_id];
        }

        return $results;
    }

    /**
     * Return region object from cache by code
     *
     * @param string $country_code country code
     * @param string $region_code  region code
     *
     * @return array
     */
    public function get_cache_region_by_code($country_code, $region_code)
    {
        $langs_names = array();
        foreach ($this->pg_language->languages as $i => $language) {
            $langs_names[] = 'lang_' . $language['id'];
        }
        $this->ci->db->select(implode(", ", array_merge($this->cache_attrs_region, $langs_names)))->from(CACHE_REGIONS_TABLE)->where("country_code",
                                                                                                                                 $country_code)->where("code",
                                                                                                                                                       $region_code);
        $data = $this->ci->db->get()->result_array();
        if (!empty($data)) {
            $lang_id = $this->pg_language->current_lang_id;
            if (!empty($data[0]['lang_' . $lang_id])) {
                $data[0]['name'] = $data[0]['lang_' . $lang_id];
            }

            return $data[0];
        }

        return array();
    }

    /**
     * Return countries objects as array
     *
     * @param array   $order_by          sorting data
     * @param array   $params            filters data
     * @param array   $filter_object_ids filters identifiers
     * @param integer $lang_id           language identifier
     *
     * @return array
     */
    public function get_countries($order_by = array(), $params = array(), $filter_object_ids = array(), $lang_id = false)
    {
        /// return installed countries

        if ($lang_id === false) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        $select_attrs = $this->attrs_country;
        if ($lang_id) {
            $select_attrs = array_diff($select_attrs, array('name'));
            $select_attrs[] = 'lang_' . $lang_id . ' as name';
        }

        $this->ci->db->select(implode(", ", $select_attrs))->from(COUNTRIES_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            $all_fields = $this->attrs_country;
            foreach ($this->ci->pg_language->languages as $lang_id => $lang_data) {
                $all_fields[] = 'lang_' . $lang_id;
            }
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $all_fields)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = array();
            foreach ($results as $r) {
                $this->country_cache[$r['code']] = $data[$r['code']] = $r;
            }

            return $data;
        }

        return array();
    }

    /**
     * Return countries objects by filters identifiers as array
     *
     * @param array   $filter_object_ids filters identifiers
     * @param integer $lang_id           language identifier
     *
     * @return array
     */
    public function get_countries_by_code($filter_object_ids = array(), $lang_id = false)
    {
        /// return installed regions

        if ($lang_id === false) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        $select_attrs = $this->attrs_country;
        if ($lang_id) {
            $select_attrs[] = 'lang_' . $lang_id . ' as name';
        }

        if (!isset($filter_object_ids) || !is_array($filter_object_ids)) {
            return array();
        }

        $data = array();

        foreach ($filter_object_ids as $index => $code) {
            if (isset($this->country_cache[$code])) {
                $data[$code] = $this->country_cache[$code];
                unset($filter_object_ids[$index]);
            }
        }

        if (empty($filter_object_ids)) {
            return $data;
        }

        $this->ci->db->select(implode(", ", $select_attrs))->from(COUNTRIES_TABLE);

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("code", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $this->country_cache[$r['code']] = $data[$r['code']] = $r;
            }
        }

        return $data;
    }

    /**
     * Return number of countries
     *
     * @param array $params filters parameters
     *
     * @return integer
     */
    public function get_countries_count($params = array())
    {
        $this->ci->db->select('COUNT(*) AS cnt')->from(COUNTRIES_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Return regions objects as array
     *
     * @param string  $country_code      country code
     * @param array   $order_by          sorting data
     * @param array   $params            filters parameters
     * @param array   $filter_object_ids filters identifiers
     * @param integer $lang_id           language identifier
     *
     * @return array
     */
    public function get_regions($country_code, $order_by = array(), $params = array(), $filter_object_ids = array(), $lang_id = false)
    {
        /// return installed regions

        if ($lang_id === false) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        $select_attrs = $this->attrs_region;
        if ($lang_id) {
            $select_attrs = array_diff($select_attrs, array('name'));
            $select_attrs[] = 'lang_' . $lang_id . ' as name';
        }

        $this->ci->db->select(implode(", ", $select_attrs))->from(REGIONS_TABLE)->where("country_code", $country_code);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            $all_fields = $this->attrs_region;
            foreach ($this->ci->pg_language->languages as $lang_id => $lang_data) {
                $all_fields[] = 'lang_' . $lang_id;
            }
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $all_fields)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = array();
            foreach ($results as $r) {
                $this->region_cache[$r['id']] = $data[$r['id']] = $r;
            }

            return $data;
        }

        return array();
    }

    /**
     * Return region objects by identifiers
     *
     * @param array   $filter_object_ids region identifiers
     * @param integer $lang_id           language identifier
     *
     * @return array
     */
    public function get_regions_by_id($filter_object_ids = array(), $lang_id = false)
    {
        /// return installed regions

        if ($lang_id === false) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        $select_attrs = $this->attrs_region;
        if ($lang_id) {
            $select_attrs[] = 'lang_' . $lang_id . ' as name';
        }

        if (!isset($filter_object_ids) || !is_array($filter_object_ids)) {
            return array();
        }

        $data = array();

        foreach ($filter_object_ids as $index => $id) {
            if (isset($this->region_cache[$id])) {
                $data[$id] = $this->region_cache[$id];
                unset($filter_object_ids[$index]);
            }
        }

        if (empty($filter_object_ids)) {
            return $data;
        }

        $this->ci->db->select(implode(", ", $select_attrs))->from(REGIONS_TABLE);

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $this->region_cache[$r["id"]] = $data[$r["id"]] = $r;
            }
        }

        return $data;
    }

    /**
     * Return region objects by code
     *
     * @param string $country_code      country code
     * @param array  $order_by          sorting data
     * @param array  $params            filters parameters
     * @param array  $filter_object_ids filters identifiers
     *
     * @return array
     */
    public function get_regions_by_code($country_code, $order_by = array(), $params = array(), $filter_object_ids = array())
    {
        $data = array();
        $regions = $this->get_regions($country_code, $order_by, $params, $filter_object_ids);
        if (!empty($regions) && is_array($regions)) {
            foreach ($regions as $r) {
                if (!empty($r["code"])) {
                    $data[$r["code"]] = $r;
                }
            }
        }

        return $data;
    }

    /**
     * Return cities objects as array
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     results per page
     * @param array   $order_by          sorting data
     * @param array   $params            filters parameters
     * @param array   $filter_object_ids filters identifiers
     * @param integer $lang_id           language identifier
     *
     * @return array
     */
    public function get_cities($page = null, $items_on_page = null, $order_by = array(), $params = array(), $filter_object_ids = array(), $lang_id = false)
    {
        /// return installed cities

        if ($lang_id === false) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        $select_attrs = $this->attrs_city;
        if ($lang_id) {
            $select_attrs = array_diff($select_attrs, array('name'));
            $select_attrs[] = 'lang_' . $lang_id . ' as name';
        }

        $this->ci->db->select(implode(", ", $select_attrs))->from(CITIES_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            $all_fields = $this->attrs_city;
            foreach ($this->ci->pg_language->languages as $lang_id => $lang_data) {
                $all_fields[] = 'lang_' . $lang_id;
            }
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $all_fields)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }
        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = array();
            foreach ($results as $r) {
                $this->city_cache[$r['id']] = $data[$r['id']] = $r;
            }

            return $data;
        }

        return array();
    }

    /**
     * Return number of cities
     *
     * @param array $params            filters parameters
     * @param array $filter_object_ids filters identifiers
     *
     * @return integer
     */
    public function get_cities_count($params = array(), $filter_object_ids = array())
    {
        /// return installed cities
        $this->ci->db->select("COUNT(*) AS cnt")->from(CITIES_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Return cities objects as array
     *
     * @param array   $filter_object_ids filters identifiers
     * @param integer $lang_id           language identifier
     *
     * @return array
     */
    public function get_cities_by_id($filter_object_ids = array(), $lang_id = false)
    {
        /// return installed cities

        if ($lang_id === false) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        $select_attrs = $this->attrs_city;
        if ($lang_id) {
            $select_attrs = array_diff($select_attrs, array('name'));
            $select_attrs[] = 'lang_' . $lang_id . ' as name';
        }

        if (!isset($filter_object_ids) || !is_array($filter_object_ids)) {
            return array();
        }

        $data = array();

        foreach ($filter_object_ids as $index => $id) {
            if (isset($this->city_cache[$id])) {
                $data[$id] = $this->city_cache[$id];
                unset($filter_object_ids[$index]);
            }
        }

        if (empty($filter_object_ids)) {
            return $data;
        }

        $this->ci->db->select(implode(", ", $select_attrs))->from(CITIES_TABLE);

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $this->city_cache[$r['id']] = $data[$r['id']] = $r;
            }
        }

        return $data;
    }

    /**
     * Return cities objects by radius
     *
     * @param float   $lat               point latitude
     * @param float   $lon               point longitude
     * @param integer $radius            search radius
     * @param string  $radius_type       radius measurement
     * @param integer $page              page of results
     * @param integer $items_on_page     results per page
     * @param array   $params            filters parameters
     * @param array   $filter_object_ids filters identifiers
     *
     * @return array
     */
    public function get_cities_by_radius($lat, $lon, $radius = 10, $radius_type = "km", $page = null, $items_on_page = null, $params = array(), $filter_object_ids = array())
    {
        $lang_id = $this->ci->pg_language->current_lang_id;

        $select_attrs = $this->attrs_city;
        $select_attrs = array_diff($select_attrs, array('name'));
        $select_attrs[] = 'lang_' . $lang_id . ' as name';

        /// return installed cities
        $this->ci->db->select(implode(", ", $select_attrs))->from(CITIES_TABLE);

        $radius = ($radius_type == "mile") ? $radius : $radius * 0.6213712;
        $this->ci->db->where('(POW((69.1*(lon-"' . $lon . '")*cos(' . $lat . '/57.3)),"2")+POW((69.1*(lat-"' . $lat . '")),"2"))<(' . ($radius * $radius) . ')');

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = array();
            foreach ($results as $r) {
                $this->city_cache[$r['id']] = $data[$r['id']] = $r;
            }

            return $data;
        }

        return array();
    }

    /**
     * Returns closest city by radius distance
     *
     * @param type $latitude
     * @param type $longitude
     * @param type $distance
     * @return type
     */
    public function get_closest_cities($latitude = 0, $longitude = 0, $distance = 10000)
    {
        $distance = intval($distance);
        
        return $this->ci->db->query("SELECT *, ((ACOS(SIN('{$latitude}' * PI() / 180) * SIN(latitude * PI() / 180) + COS('{$latitude}' * PI() / 180) * COS(latitude * PI() / 180) * COS(('{$longitude}' - longitude) * PI() / 180)) * 180 / PI()) * 60 * 1.1515) AS `distance` FROM `" . CITIES_TABLE . "` HAVING `distance`<='10000' ORDER BY `distance` ASC LIMIT 0 , 10")->result_array();
        
     }

    /**
     * Return country object by code
     *
     * @param string  $country_code country code
     * @param integer $lang_id      language identifier
     * @param array   $languages    languages data
     *
     * @return array
     */
    public function get_country($country_code, $lang_id = false, $languages = array())
    {
        /// return installed country

        $cache_enabled = false;

        if ($lang_id === false) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        $select_attrs = $this->attrs_country;
        if ($lang_id) {
            $select_attrs = array_diff($select_attrs, array('name'));
            $select_attrs[] = 'lang_' . $lang_id . ' as name';
            $cache_enabled = true;
        }
        if (is_array($languages) && count($languages) > 0) {
            foreach ($languages as $id => $value) {
                $select_attrs[] = 'lang_' . $id . ' as lang_' . $id;
            }
            $cache_enabled = false;
        }

        if ($cache_enabled && isset($this->country_cache[$country_code])) {
            return $this->country_cache[$country_code];
        }

        $this->ci->db->select(implode(", ", $select_attrs))->from(COUNTRIES_TABLE)->where("code", $country_code);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            if ($cache_enabled) {
                $this->country_cache[$country_code] = $results[0];
            }

            return $results[0];
        }

        return array();
    }

    /**
     * Return country object by identifier
     *
     * @param integer $country_id country identifier
     *
     * @return array
     */
    public function get_country_by_id($country_id)
    {
        /// return installed country
        $this->ci->db->select(implode(", ", $this->attrs_country))
                ->from(COUNTRIES_TABLE)
                ->where("id", $country_id)
                ->or_where('code', $country_id)
                ->limit(1);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $results[0];
        }

        return array();
    }

    /**
     * Return region object by identifier
     *
     * @param integer $region_id region identifier
     * @param integer $lang_id   language identifier
     * @param array   $languages languages data
     *
     * @return array
     */
    public function get_region($region_id, $lang_id = false, $languages = array())
    {
        /// return installed region

        $cache_enabled = false;

        if ($lang_id === false) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        $select_attrs = $this->attrs_region;
        if ($lang_id) {
            $select_attrs = array_diff($select_attrs, array('name'));
            $select_attrs[] = 'lang_' . $lang_id . ' as name';
            $cache_enabled = true;
        }
        if (is_array($languages) && count($languages) > 0) {
            foreach ($languages as $id => $value) {
                $select_attrs[] = 'lang_' . $id . ' as lang_' . $id;
            }
            $cache_enabled = false;
        }

        if ($cache_enabled && isset($this->region_cache[$region_id])) {
            return $this->region_cache[$region_id];
        }

        $this->ci->db->select(implode(", ", $select_attrs))->from(REGIONS_TABLE)->where("id", $region_id);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            if ($cache_enabled) {
                $this->region_cache[$region_id] = $results[0];
            }

            return $results[0];
        }

        return array();
    }

    /**
     * Return region object by code
     *
     * @param string $region_code  region code
     * @param string $country_code country code
     *
     * @return array
     */
    public function get_region_by_code($region_code, $country_code = null)
    {
        /// return installed region
        $this->ci->db->select(implode(", ", $this->attrs_region));
        $this->ci->db->from(REGIONS_TABLE);
        if ($country_code) {
            $this->ci->db->where("country_code", $country_code);
        }
        $this->ci->db->where("code", $region_code);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $results[0];
        }

        return array();
    }

    /**
     * Return city object by identifier
     *
     * @param integer $city_id   city identifier
     * @param integer $lang_id   language identifier
     * @param array   $languages languages data
     *
     * @return array
     */
    public function get_city($city_id, $lang_id = false, $languages = array())
    {
        /// return installed city

        $cache_enabled = false;

        if ($lang_id === false) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        $select_attrs = $this->attrs_city;
        if ($lang_id) {
            $select_attrs = array_diff($select_attrs, array('name'));
            $select_attrs[] = 'lang_' . $lang_id . ' as name';
            $cache_enabled = true;
        }

        if (is_array($languages) && count($languages) > 0) {
            foreach ($languages as $id => $value) {
                $select_attrs[] = 'lang_' . $id . ' as lang_' . $id;
            }
            $cache_enabled = false;
        }

        if ($cache_enabled && isset($this->city_cache[$city_id])) {
            return $this->city_cache[$city_id];
        }

        $this->ci->db->select(implode(", ", $select_attrs))->from(CITIES_TABLE)->where("id", $city_id);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            if ($cache_enabled) {
                $this->city_cache[$city_id] = $results[0];
            }

            return $results[0];
        }

        return array();
    }

    /**
     * Return location objects as array
     *
     * @param string  $loc_name  location name
     * @param array   $order_by  sorting data
     * @param integer $lang_id   language identifier
     * @param array   $languages languages data
     * @param integer $country   country identifier
     * @param integer $region    region identifier
     * @param integer $city      city identifier
     *
     * @return array
     */
    public function get_locations($loc_name, $order_by = array(), $lang_id = false, $languages = array(), $country = null, $region = null, $city = null)
    {
        $return = array();

        if (!is_array($loc_name)) {
            $loc_name = trim($loc_name);
        }
        if ($loc_name) {
            $loc_names = explode(' ', $loc_name);
            $where_str = '';
            if (count($loc_names) > 0) {
                foreach ($loc_names as $loc_name) {
                    if (empty($loc_name)) {
                        continue;
                    }
                    $where_str .= strlen($where_str) > 0 ? " OR name like " . $this->ci->db->escape('%' . $loc_name . '%') . " " : " name like " . $this->ci->db->escape('%' . $loc_name . '%') . " ";
                    if (is_array($languages) && count($languages) > 0) {
                        foreach ($languages as $lid => $lang_value) {
                            $where_str .= " OR lang_" . $lid . " like " . $this->ci->db->escape('%' . $loc_name . '%') . " ";
                        }
                    }
                }
            }
        }

        // Search in countries
        if (!$country) {
            $select_attrs = $this->attrs_country;
            if ($lang_id) {
                $select_attrs = array_diff($select_attrs, array('name'));
                $select_attrs[] = 'lang_' . $lang_id . ' as name';
            }

            $this->ci->db->select(implode(", ", $select_attrs))->from(COUNTRIES_TABLE);

            if ($where_str) {
                $this->ci->db->where($where_str, null, false);
            }

            $this->ci->db->limit(3);
            if (is_array($order_by) && count($order_by) > 0) {
                foreach ($order_by as $field => $dir) {
                    if (in_array($field, $this->attrs_country)) {
                        $this->ci->db->order_by($field . " " . $dir);
                    }
                }
            }

            $results = $this->ci->db->get()->result_array();
            $return['countries'] = $results ? $results : array();

            if (!empty($return['countries'])) {
                $this->ci->load->helper('countries');
                $locations_ids = array();

                foreach ($return['countries'] as $key => $row_countries) {
                    $locations_ids[$key] = array(
                        'country' => $row_countries['code'],
                    );

                    $list_locations = countries_output_format($locations_ids);
                    $return['countries'][$key]['name'] = $list_locations[$key];
                }
            }
        } else {
            $return['countries'] = array();
        }

        // Search in regions
        if (!$region) {
            $select_attrs = $this->attrs_region;
            if ($lang_id) {
                $select_attrs = array_diff($select_attrs, array('name'));
                $select_attrs[] = 'lang_' . $lang_id . ' as name';
            }

            $this->ci->db->select(implode(", ", $select_attrs))->from(REGIONS_TABLE);
            if ($country) {
                $this->ci->db->where('country_code', $country);
            }
            if ($where_str) {
                $this->ci->db->where('(' . $where_str . ')', null, false);
            }
            $this->ci->db->limit(3);

            if (is_array($order_by) && count($order_by) > 0) {
                foreach ($order_by as $field => $dir) {
                    if (in_array($field, $this->attrs_region)) {
                        $this->ci->db->order_by($field . " " . $dir);
                    }
                }
            }
            $results = $this->ci->db->get()->result_array();
            $return['regions'] = $results ? $results : array();

            if (!empty($return['regions'])) {
                $this->ci->load->helper('countries');
                $locations_ids = array();

                foreach ($return['regions'] as $key => $row_region) {
                    $locations_ids[$key] = array(
                        'country' => $row_region['country_code'],
                        'region' => $row_region['id'],
                    );

                    $list_locations = regions_output_format($locations_ids);
                    $return['regions'][$key]['name'] = $list_locations[$key];
                }
            }
        } else {
            $return['regions'] = array();
        }

        // Search in cities
        if (!$city) {
            $select_attrs = $this->attrs_city;
            if ($lang_id) {
                $select_attrs = array_diff($select_attrs, array('name'));
                $select_attrs[] = 'lang_' . $lang_id . ' as name';
            }
            $this->ci->db->select(implode(", ", $select_attrs))->from(CITIES_TABLE);
            if ($country) {
                $this->ci->db->where('country_code', $country);
            }
            if ($region) {
                $this->ci->db->where('id_region', $region);
            }
            if ($where_str) {
                $this->ci->db->where('(' . $where_str . ')', null, false);
            }
            $this->ci->db->limit(3);
            if (is_array($order_by) && count($order_by) > 0) {
                foreach ($order_by as $field => $dir) {
                    if (in_array($field, $this->attrs_city)) {
                        $this->ci->db->order_by($field . " " . $dir);
                    }
                }
            }
            $results = $this->ci->db->get()->result_array();
            $return['cities'] = $results ? $results : array();

            if (!empty($return['cities'])) {
                $this->ci->load->helper('countries');
                $locations_ids = array();

                foreach ($return['cities'] as $key => $row_cities) {
                    $locations_ids[$key] = array(
                        'country' => $row_cities['country_code'],
                        'region' => $row_cities['id_region'],
                        'city' => $row_cities['id'],
                    );

                    $list_locations = cities_output_format($locations_ids);
                    $return['cities'][$key]['name'] = $list_locations[$key];
                }
            }
        } else {
            $return['cities'] = array();
        }

        // Search in districts
        $use_district = $this->ci->pg_module->get_module_config('countries', 'use_district');
        if ($use_district) {
            $select_attrs = $this->attrs_district;
            if ($lang_id) {
                $select_attrs = array_diff($select_attrs, array('name'));
                $select_attrs[] = 'lang_' . $lang_id . ' as name';
            }
            $this->ci->db->select(implode(", ", $select_attrs))->from(DISTRICTS_TABLE);
            if ($country) {
                $this->ci->db->where('country_code', $country);
            }
            if ($region) {
                $this->ci->db->where('id_region', $region);
            }
            if ($city) {
                $this->ci->db->where('id_city', $city);
            }
            if ($where_str) {
                $this->ci->db->where('(' . $where_str . ')', null, false);
            }
            $this->ci->db->limit(3);
            if (is_array($order_by) && count($order_by) > 0) {
                foreach ($order_by as $field => $dir) {
                    if (in_array($field, $this->attrs_city)) {
                        $this->ci->db->order_by($field . " " . $dir);
                    }
                }
            }
            $results = $this->ci->db->get()->result_array();
            $return['districts'] = $results ? $results : array();

            if (!empty($return['districts'])) {
                $this->ci->load->helper('countries');
                $locations_ids = array();

                foreach ($return['districts'] as $key => $row_districts) {
                    $locations_ids[$key] = array(
                        'country' => $row_districts['country_code'],
                        'region' => $row_districts['id_region'],
                        'city' => $row_districts['id_city'],
                        'district' => $row_districts['id'],
                    );

                    $list_locations = districts_output_format($locations_ids);
                    $return['districts'][$key]['name'] = $list_locations[$key];
                }
            }
        } else {
            $return['districts'] = array();
        }

        return $return;
    }

    /**
     * Install cities
     *
     * @param string $country_code country code
     * @param string $region_code  region code
     * @param array  $languages    languages data
     *
     * @return boolean
     */
    public function install_cities($country_code, $region_code, $languages = array())
    {
        /// set country installed
        /// set region installed
        /// get region data(new id and region code)
        /// while wget_cities !== true||false => wget_cities and install sities (use_infile_city_install)

        $country_data = $this->get_country($country_code);
        if (!$country_data) {
            $country_data = $this->get_cache_country_by_code($country_code);
            if (empty($country_data)) {
                return false;
            }
            $insert_data = array(
                'code' => $country_code,
                'name' => $country_data["name"],
                'areainsqkm' => $country_data["areainsqkm"],
                'continent' => $country_data["continent"],
                'currency' => $country_data["currency"],
                "priority" => $this->get_country_max_priority(),
            );
            if (is_array($languages) && count($languages) > 0) {
                foreach ($languages as $id => $value) {
                    $insert_data['lang_' . $id] = $country_data["name"];
                }
            }
            $this->save_country($country_code, $insert_data, "add");
        }

        $cache_region_data = $this->get_cache_region_by_code($country_code, $region_code);
        if (!$cache_region_data) {
            return false;
        }
        $region_server_id = $cache_region_data["id_region"];

        $region_data = $this->get_region_by_code($region_code, $country_code);
        if (!$region_data) {
            $insert_data = array(
                'country_code' => $country_code,
                'code' => $region_code,
                'name' => $cache_region_data["name"],
                "priority" => $this->get_region_max_priority($country_code),
            );
            if (is_array($languages) && count($languages) > 0) {
                foreach ($languages as $id => $value) {
                    $insert_data['lang_' . $id] = $cache_region_data['lang_' . $id];
                }
            }
            $id_region = $this->save_region(null, $insert_data);
            $region_data = $this->get_region($id_region);
        }

        $this->ci->db->where("id_region", $region_data["id"]);
        $this->ci->db->delete(CITIES_TABLE);

        $install_data = 'start';
        $max_itaration = 20;
        $itaration_counter = 0;

        if ($this->use_infile_city_install && is_array($languages) && count($languages) > 0) {
            $sql_query = "UPDATE " . CITIES_TABLE . " SET ";
            $sql_set = "";
            $sql_where = "";
            foreach ($languages as $id => $value) {
                $sql_set .= strlen($sql_set) > 0 ? ", lang_" . $id . " = name" : "lang_" . $id . " = name";
                $sql_where .= strlen($sql_where) ? " AND lang_" . $id . " = '' " : " lang_" . $id . " = '' ";
            }
            $languages_query = $sql_query . $sql_set . " WHERE " . $sql_where;

            if (isset($region_data["id"])) {
                $languages_query .= " AND id_region = " . $region_data["id"];
            }
        }

        while ($install_data !== true && $install_data !== false) {
            if ($itaration_counter > $max_itaration) {
                break;
            }
            $install_data = $this->wget_cities($country_code, $region_server_id, $region_data);
            ++$itaration_counter;
            if ($install_data !== true && $install_data !== false) {
                if ($this->use_infile_city_install) {
                    if (file_exists($install_data["file"])) {
                        $sql_result = $this->ci->db->simple_query("LOAD DATA INFILE '" . $install_data["file"] . "' INTO TABLE " . CITIES_TABLE . " CHARACTER SET utf8  FIELDS TERMINATED BY '\t';");
                        if ($sql_result === false) {
                            $this->use_infile_city_install = false;
                        }
                    }
                }

                if (!$this->use_infile_city_install) {
                    $counter = 0;
                    $data_count = count($install_data["data"]);
                    $start_sql = "INSERT INTO " . CITIES_TABLE . " (id_region, name, latitude, longitude, country_code, region_code";
                    if (is_array($languages) && count($languages) > 0) {
                        foreach ($languages as $id => $value) {
                            $start_sql .= ", lang_" . $id;
                        }
                    }
                    $start_sql .= ") VALUES ";

                    while ($counter < $data_count) {
                        unset($strings);
                        $temp_geo = array_slice($install_data["data"], $counter, $this->city_install_step);
                        foreach ($temp_geo as $incity) {
                            $string = "(" . $this->ci->db->escape($region_data["id"]) . ", " .
                                            $this->ci->db->escape(trim($incity["name"])) . ", " .
                                            $this->ci->db->escape($incity["latitude"]) . ", " .
                                            $this->ci->db->escape($incity["longitude"]) . ", " .
                                            $this->ci->db->escape($country_code) . ", " .
                                            $this->ci->db->escape($region_code);
                            if (is_array($languages) && count($languages) > 0) {
                                foreach ($languages as $id => $value) {
                                    $string .= ", " . $this->ci->db->escape(trim($incity["lang_" . $id]));
                                }
                            }
                            $string .= ")";
                            $strings[] = $string;
                        }
                        $query = $start_sql . implode(", ", $strings);
                        $this->ci->db->query($query);
                        $counter = $counter + $this->city_install_step;
                    }
                }
            }
        }

        if ($this->use_infile_city_install && !empty($languages_query)) {
            $this->ci->db->query($languages_query);
        }

        return true;
    }

    /**
     * Save country object to data source
     *
     * @param string $country_code country code
     * @param array  $data         country data
     * @param string $type         operation type (add, update)
     * @param array  $langs        languages data
     *
     * @return void
     */
    public function save_country($country_code, $data, $type = "add", $langs = array())
    {
        if (is_array($langs) && count($langs) > 0) {
            foreach ($langs as $id => $value) {
                $data['lang_' . $id] = $value;
            }
        }
        if ($type == "add") {
            if ($country_code) {
                $data["code"] = $country_code;
            }
            $this->ci->db->insert(COUNTRIES_TABLE, $data);
        } else {
            $this->ci->db->where('code', $country_code);
            $this->ci->db->update(COUNTRIES_TABLE, $data);
        }
    }

    /**
     * Save region object to data source
     *
     * @param integer $region_id region identifier
     * @param array   $data      region data
     * @param array   $langs     languages data
     *
     * @return integer
     */
    public function save_region($region_id, $data, $langs = array())
    {
        if (is_array($langs) && count($langs) > 0) {
            foreach ($langs as $id => $value) {
                $data['lang_' . $id] = $value;
            }
        }
        if (empty($region_id)) {
            $this->ci->db->insert(REGIONS_TABLE, $data);
            $region_id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $region_id);
            $this->ci->db->update(REGIONS_TABLE, $data);
        }

        return $region_id;
    }

    /**
     * Save city object to data source
     *
     * @param integer $city_id city identifier
     * @param array   $data    city data
     * @param array   $langs   languages data
     *
     * @return integer
     */
    public function save_city($city_id, $data, $langs = array())
    {
        if (is_array($langs) && count($langs) > 0) {
            foreach ($langs as $id => $value) {
                $data['lang_' . $id] = $value;
            }
        }
        if (empty($city_id)) {
            $this->ci->db->insert(CITIES_TABLE, $data);
            $city_id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $city_id);
            $this->ci->db->update(CITIES_TABLE, $data);
        }

        return $city_id;
    }

    /**
     * Remove country object by code
     *
     * @param string $country_code country code
     *
     * @return void
     */
    public function delete_country($country_code)
    {
        $this->ci->db->where('code', $country_code);
        $this->ci->db->delete(COUNTRIES_TABLE);

        $this->ci->db->where('country_code', $country_code);
        $this->ci->db->delete(REGIONS_TABLE);

        $this->ci->db->where('country_code', $country_code);
        $this->ci->db->delete(CITIES_TABLE);
    }

    /**
     * Remove region object by identifier
     *
     * @param integer $region_id region identifier
     *
     * @return void
     */
    public function delete_region($region_id)
    {
        $this->ci->db->where('id', $region_id);
        $this->ci->db->delete(REGIONS_TABLE);

        $this->ci->db->where('id_region', $region_id);
        $this->ci->db->delete(CITIES_TABLE);
    }

    /**
     * Remove city object by identifier
     *
     * @param integer $city_id city identifier
     *
     * @return void
     */
    public function delete_city($city_id)
    {
        $this->ci->db->where('id', $city_id);
        $this->ci->db->delete(CITIES_TABLE);
    }

    /**
     * Validate location data
     *
     * @param string  $type       location type (country, region, city)
     * @param integer $id         location identifier
     * @param array   $data       location data
     * @param array   $langs_data languages data
     */
    public function validate($type, $id, $data, $langs_data = false)
    {
        $return = array("errors" => array(), "data" => array(), 'langs' => array());

        if ($type == "country") {
            if (empty($id) && empty($data["code"])) {
                $return["errors"][] = l('error_code_empty', 'countries');
            } elseif (isset($data["code"])) {
                $return["data"]["code"] = strtoupper(substr(strip_tags($data["code"]), 0, 2));

                if (empty($id) || $id != $return["data"]["code"]) {
                    $params["where"]["code"] = $return["data"]["code"];
                    $countries = $this->get_countries(array(), $params);
                    if (!empty($countries)) {
                        $return["errors"][] = l('error_code_already_exists', 'countries');
                    }
                }
            }
        }

        if ($type == "region") {
            if (isset($data["country_code"])) {
                $return["data"]["country_code"] = strtoupper(substr(strip_tags($data["country_code"]), 0, 2));
                if (empty($return["data"]["country_code"])) {
                    $return["errors"][] = l('error_code_empty', 'countries');
                }
            }

            if (isset($data["code"])) {
                $return["data"]["code"] = strip_tags($data["code"]);
                if (empty($return["data"]["code"])) {
                    $return["errors"][] = l('error_region_code_empty', 'countries');
                }
            }
        }

        if ($type == "city") {
            if (isset($data["country_code"])) {
                $return["data"]["country_code"] = strtoupper(substr(strip_tags($data["country_code"]), 0, 2));
                if (empty($return["data"]["country_code"])) {
                    $return["errors"][] = l('error_code_empty', 'countries');
                }
            }
            if (isset($data["region_code"])) {
                $return["data"]["region_code"] = strip_tags($data["region_code"]);
                if (empty($return["data"]["region_code"])) {
                    $return["errors"][] = l('error_region_code_empty', 'countries');
                }
            }
            if (isset($data["id_region"])) {
                $return["data"]["id_region"] = intval($data["id_region"]);
                if (empty($return["data"]["id_region"])) {
                    $return["errors"][] = l('error_region_empty', 'countries');
                }
            }
            if (isset($data["latitude"])) {
                $return["data"]["latitude"] = strval($data["latitude"]);
            }
            if (isset($data["longitude"])) {
                $return["data"]["longitude"] = strval($data["longitude"]);
            }
        }

        if ($type == "district") {
            if (isset($data["country_code"])) {
                $return["data"]["country_code"] = strtoupper(substr(strip_tags($data["country_code"]), 0, 2));
                if (empty($return["data"]["country_code"])) {
                    $return["errors"][] = l('error_code_empty', 'countries');
                }
            }
            if (isset($data["region_code"])) {
                $return["data"]["region_code"] = strip_tags($data["region_code"]);
                if (empty($return["data"]["region_code"])) {
                    $return["errors"][] = l('error_region_code_empty', 'countries');
                }
            }
            if (isset($data["id_region"])) {
                $return["data"]["id_region"] = intval($data["id_region"]);
                if (empty($return["data"]["id_region"])) {
                    $return["errors"][] = l('error_region_empty', 'countries');
                }
            }
            if (isset($data["id_city"])) {
                $return["data"]["id_city"] = intval($data["id_city"]);
                if (empty($return["data"]["id_city"])) {
                    $return["errors"][] = l('error_city_empty', 'countries');
                }
            }
            if (isset($data["latitude"])) {
                $return["data"]["latitude"] = strval($data["latitude"]);
            }
            if (isset($data["longitude"])) {
                $return["data"]["longitude"] = strval($data["longitude"]);
            }
        }

        if (isset($data["name"])) {
            $return["data"]["name"] = strip_tags($data["name"]);
            if (empty($return["data"]["name"])) {
                $return["errors"][] = l('error_name_empty', 'countries');
            }
        }

        if (is_array($langs_data) && !empty($langs_data)) {
            $default_lang_id = $this->ci->pg_language->current_lang_id;
            if (!isset($langs_data[$default_lang_id])) {
                if (!isset($data["name"]) || !empty($return["data"]["name"])) {
                    $return['errors'][] = l('error_name_empty', "countries");
                }
            } else {
                $return['langs'][$default_lang_id] = trim(strip_tags($langs_data[$default_lang_id]));
                if (empty($return['langs'][$default_lang_id])) {
                    if (!isset($data["name"]) || !empty($return["data"]["name"])) {
                        $return['errors'][] = l('error_name_empty', "countries");
                    }
                } else {
                    foreach ($this->ci->pg_language->languages as $lid => $lang_data) {
                        if ($lid == $default_lang_id) {
                            continue;
                        }
                        if (!isset($langs_data[$lid])) {
                            $return['langs'][$lid] = $return['langs'][$default_lang_id];
                        } else {
                            $return['langs'][$lid] = trim(strip_tags($langs_data[$lid]));
                            if (empty($return['langs'][$lid])) {
                                $return['langs'][$lid] = $return['langs'][$default_lang_id];
                            }
                        }
                    }
                }
            }
        }

        return $return;
    }

    /**
     * Return max priority of countries
     *
     * @return integer
     */
    public function get_country_max_priority()
    {
        $this->ci->db->select("MAX(priority) as max_priority")->from(COUNTRIES_TABLE);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["max_priority"]);
        }

        return 0;
    }

    /**
     * Return max priority of regions
     *
     * @param string $country_code country code
     *
     * @return integer
     */
    public function get_region_max_priority($country_code)
    {
        $this->ci->db->select("MAX(priority) as max_priority")->from(REGIONS_TABLE)->where("country_code", $country_code);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["max_priority"]);
        }

        return 0;
    }

    /**
     * Save country object priority
     *
     * @param string  $country_code country code
     * @param integer $priority     priority value
     *
     * @return void
     */
    public function set_country_priority($country_code, $priority)
    {
        $data["priority"] = intval($priority);
        $data["sorted"] = 1;
        $this->ci->db->where("code", $country_code);
        $this->ci->db->update(COUNTRIES_TABLE, $data);
    }

    /**
     * Save region object priority
     *
     * @param string  $region_id region identifier
     * @param integer $priority  priority value
     *
     * @return void
     */
    public function set_region_priority($region_id, $priority)
    {
        $data["priority"] = intval($priority);
        $data["sorted"] = 1;
        $this->ci->db->where("id", $region_id);
        $this->ci->db->update(REGIONS_TABLE, $data);
    }

    /**
     * Save city object priority
     *
     * @param string  $city_id  city identifier
     * @param integer $priority priority value
     *
     * @return void
     */
    public function set_city_priority($city_id, $priority)
    {
        $data["priority"] = intval($priority);
        $data["sorted"] = 1;
        $this->ci->db->where("id", $city_id);
        $this->ci->db->update(CITIES_TABLE, $data);
    }

    /**
     * Set countries default priority
     *
     * @param array $countries_array
     *
     * @return void
     */
    public function set_default_priority_country($countries_array)
    {
        $data["priority"] = sizeof($countries_array) + 1;
        $data["sorted"] = 0;
        if ($data["priority"] > 1) {
            $this->ci->db->where_not_in("code", $countries_array);
        }
        $this->ci->db->update(COUNTRIES_TABLE, $data);
    }

    /**
     * Set regions default priority
     *
     * @param string $country_code  country code
     * @param array  $regions_array
     *
     * @return void
     */
    public function set_default_priority_region($country_code, $regions_array)
    {
        $data["priority"] = sizeof($regions_array) + 1;
        $data["sorted"] = 0;
        $this->ci->db->where("country_code", $country_code);
        if ($data["priority"] > 1) {
            $this->ci->db->where_not_in("id", $regions_array);
        }
        $this->ci->db->update(REGIONS_TABLE, $data);
    }

    /**
     * Set cities default priority
     *
     * @param string $country_code country code
     * @param string $id_region    identifier
     * @param array  $cities_array
     *
     * @return void
     */
    public function set_default_priority_city($country_code, $id_region, $cities_array)
    {
        $data["priority"] = sizeof($cities_array) + 1;
        $data["sorted"] = 0;
        $this->ci->db->where("country_code", $country_code);
        $this->ci->db->where("id_region", $id_region);
        if ($data["priority"] > 1) {
            $this->ci->db->where_not_in("id", $cities_array);
        }
        $this->ci->db->update(CITIES_TABLE, $data);
    }

    /**
     * Set districts default priority
     *
     * @param string $country_code    country code
     * @param string $id_region       identifier
     * @param string $id_city         identifier
     * @param array  $districts_array
     *
     * @return void
     */
    public function set_default_priority_district($country_code, $id_region, $id_city, $districts_array)
    {
        $data["priority"] = sizeof($districts_array) + 1;
        $data["sorted"] = 0;
        $this->ci->db->where("country_code", $country_code);
        $this->ci->db->where("id_region", $id_region);
        $this->ci->db->where("id_city", $id_city);
        if ($data["priority"] > 1) {
            $this->ci->db->where_not_in("id", $districts_array);
        }
        $this->ci->db->update(DISTRICTS_TABLE, $data);
    }

    /**
     * Processing events on adding new languages
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function lang_dedicate_module_callback_add($lang_id = false)
    {
        if ($lang_id) {
            $this->ci->load->dbforge();
            $fields["lang_" . $lang_id] = array(
                'type' => 'VARCHAR',
                'constraint' => '255',
                'null' => false,
            );
            $default_lang_id = $this->ci->pg_language->get_default_lang_id();

            // Add contries column
            $exists_fields = $this->ci->db->list_fields(COUNTRIES_TABLE);
            $this->ci->dbforge->add_column(COUNTRIES_TABLE, $fields);
            // Set default names
            if (in_array("lang_" . $default_lang_id, $exists_fields)) {
                $this->ci->db->set('lang_' . $lang_id, 'lang_' . $default_lang_id, false);
            } else {
                $this->ci->db->set('lang_' . $lang_id, 'name', false);
            }
            $this->ci->db->update(COUNTRIES_TABLE);

            // Add regions column
            $exists_fields = $this->ci->db->list_fields(REGIONS_TABLE);
            $this->ci->dbforge->add_column(REGIONS_TABLE, $fields);
            // Set default name
            if (in_array("lang_" . $default_lang_id, $exists_fields)) {
                $this->ci->db->set('lang_' . $lang_id, 'lang_' . $default_lang_id, false);
            } else {
                $this->ci->db->set('lang_' . $lang_id, 'name', false);
            }
            $this->ci->db->update(REGIONS_TABLE);

            // Add cities column
            $exists_fields = $this->ci->db->list_fields(CITIES_TABLE);
            $this->ci->dbforge->add_column(CITIES_TABLE, $fields);
            // Set default names
            if (in_array("lang_" . $default_lang_id, $exists_fields)) {
                $this->ci->db->set('lang_' . $lang_id, 'lang_' . $default_lang_id, false);
            } else {
                $this->ci->db->set('lang_' . $lang_id, 'name', false);
            }
            $this->ci->db->update(CITIES_TABLE);

            // Add districts column
            $exists_fields = $this->ci->db->list_fields(DISTRICTS_TABLE);
            $this->ci->dbforge->add_column(DISTRICTS_TABLE, $fields);
            // Set default names
            if (in_array("lang_" . $default_lang_id, $exists_fields)) {
                $this->ci->db->set('lang_' . $lang_id, 'lang_' . $default_lang_id, false);
            } else {
                $this->ci->db->set('lang_' . $lang_id, 'name', false);
            }
            $this->ci->db->update(DISTRICTS_TABLE);

            // Add contries cache column
            $exists_fields = $this->ci->db->list_fields(CACHE_COUNTRIES_TABLE);
            $this->ci->dbforge->add_column(CACHE_COUNTRIES_TABLE, $fields);
            // Set default names
            if (in_array("lang_" . $default_lang_id, $exists_fields)) {
                $this->ci->db->set('lang_' . $lang_id, 'lang_' . $default_lang_id, false);
            } else {
                $this->ci->db->set('lang_' . $lang_id, 'name', false);
            }
            $this->ci->db->update(CACHE_COUNTRIES_TABLE);

            // Add regions cache column
            $exists_fields = $this->ci->db->list_fields(CACHE_REGIONS_TABLE);
            $this->ci->dbforge->add_column(CACHE_REGIONS_TABLE, $fields);
            // Set default name
            if (in_array("lang_" . $default_lang_id, $exists_fields)) {
                $this->ci->db->set('lang_' . $lang_id, 'lang_' . $default_lang_id, false);
            } else {
                $this->ci->db->set('lang_' . $lang_id, 'name', false);
            }
            $this->ci->db->update(CACHE_REGIONS_TABLE);
        }
    }

    /**
     * Processing events on removing language
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function lang_dedicate_module_callback_delete($lang_id = false)
    {
        if ($lang_id) {
            $field_name = "lang_" . $lang_id;
            $this->ci->load->dbforge();

            // Delete countries column
            $fields_exists = $this->ci->db->list_fields(COUNTRIES_TABLE);
            if (in_array("lang_" . $lang_id, $fields_exists)) {
                $this->ci->dbforge->drop_column(COUNTRIES_TABLE, $field_name);
            }

            // Delete regions column
            $fields_exists = $this->ci->db->list_fields(REGIONS_TABLE);
            if (in_array("lang_" . $lang_id, $fields_exists)) {
                $this->ci->dbforge->drop_column(REGIONS_TABLE, $field_name);
            }

            // Delete cities column
            $fields_exists = $this->ci->db->list_fields(CITIES_TABLE);
            if (in_array("lang_" . $lang_id, $fields_exists)) {
                $this->ci->dbforge->drop_column(CITIES_TABLE, $field_name);
            }

            // Delete districts column
            $fields_exists = $this->ci->db->list_fields(DISTRICTS_TABLE);
            if (in_array("lang_" . $lang_id, $fields_exists)) {
                $this->ci->dbforge->drop_column(DISTRICTS_TABLE, $field_name);
            }

            // Delete countries cache column
            $fields_exists = $this->ci->db->list_fields(CACHE_COUNTRIES_TABLE);
            if (in_array("lang_" . $lang_id, $fields_exists)) {
                $this->ci->dbforge->drop_column(CACHE_COUNTRIES_TABLE, $field_name);
            }

            // Delete regions column
            $fields_exists = $this->ci->db->list_fields(CACHE_REGIONS_TABLE);
            if (in_array("lang_" . $lang_id, $fields_exists)) {
                $this->ci->dbforge->drop_column(CACHE_REGIONS_TABLE, $field_name);
            }
        }
    }

    /**
     * Return country object by name
     *
     * @param string  $country_name country name
     * @param integer $lang_id      language identifier
     *
     * @return array
     */
    public function get_country_by_name($country_name, $lang_id = false)
    {
        /// return installed country
        if (!$lang_id) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }
        $this->ci->db->select(implode(", ", $this->attrs_country))->from(COUNTRIES_TABLE)->where("lang_" . $lang_id,
                                                                                             $country_name);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $results[0];
        }

        return array();
    }

    /**
     * Return region object by name
     *
     * @param string  $region_name region name
     * @param string  $country_id  country code
     * @param integer $lang_id     language identifier
     *
     * @return array
     */
    public function get_region_by_name($region_name, $country_id = "", $lang_id = false)
    {
        /// return installed region
        if (!$lang_id) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }
        $this->ci->db->select(implode(", ", $this->attrs_region))->from(REGIONS_TABLE)->where("lang_" . $lang_id,
                                                                                          $region_name);
        if ($country_id) {
            $this->ci->db->where("country_code", $country_id);
        }
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $results[0];
        }

        return array();
    }

    /**
     * Return city object by name
     *
     * @param string  $city_name  city name
     * @param integer $region_id  region identifier
     * @param string  $country_id country code
     * @param integer $lang_id    language identifier
     *
     * @return array
     */
    public function get_city_by_name($city_name, $region_id = 0, $country_id = "", $lang_id = false)
    {
        /// return installed city
        if (!$lang_id) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }
        $this->ci->db->select(implode(", ", $this->attrs_city))->from(CITIES_TABLE)->where('lang_' . $lang_id, $city_name);
        if ($region_id) {
            $this->ci->db->where("id_region", $region_id);
        }
        if ($country_id) {
            $this->ci->db->where("country_code", $country_id);
        }
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $results[0];
        }

        return array();
    }

    /**
     * Parse location data from string
     *
     * @param strign $location location string
     *
     * @return array
     */
    public function get_location_from_string($location = '')
    {
        $return = array('keyword' => '');

        if (empty($location)) {
            return $return;
        }

        if (!$lang_id) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        $location = explode(',', $location);

        $attrs_country = $this->attrs_country;
        $attrs_country[] = "lang_" . $lang_id;
        $this->ci->db->select(implode(", ", $attrs_country))->from(COUNTRIES_TABLE)->where_in("lang_" . $lang_id, $location);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = $results[0];
            if (($key = array_search($data["lang_" . $lang_id], $location)) !== false) {
                unset($location[$key]);
            }
            $return['country'] = $data['code'];
        }

        if (empty($location)) {
            return $return;
        }

        $attrs_region = $this->attrs_region;
        $attrs_region[] = "lang_" . $lang_id;
        $this->ci->db->select(implode(", ", $attrs_region))->from(REGIONS_TABLE)->where_in("lang_" . $lang_id, $location);
        if (isset($return['country'])) {
            $this->ci->db->where("country_code", $return['country']);
        }
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = $results[0];
            if (($key = array_search($data["lang_" . $lang_id], $location)) !== false) {
                unset($location[$key]);
            }
            $return['region'] = $data['id'];
            $return['country'] = $data['country_code'];
        }

        if (empty($location)) {
            return $return;
        }

        $attrs_city = $this->attrs_city;
        $attrs_city[] = "lang_" . $lang_id;
        $this->ci->db->select(implode(", ", $attrs_city))->from(CITIES_TABLE)->where_in('lang_' . $lang_id, $location);
        if (isset($return['country'])) {
            $this->ci->db->where("country_code", $return['country']);
        }
        if (isset($return['region'])) {
            $this->ci->db->where("id_region", $return['region']);
        }
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = $results[0];
            if (($key = array_search($data["lang_" . $lang_id], $location)) !== false) {
                unset($location[$key]);
            }
            $retrun['city'] = $data['id'];
            $return['region'] = $data['id_region'];
            $return['country'] = $data['country_code'];
        }

        if (!empty($location)) {
            $return['keyword'] = implode(',', $location);
        }

        return $return;
    }

    /**
     * Remove district object by identifier
     *
     * @param integer $district_id district identifier
     *
     * @return void
     */
    public function delete_district($district_id)
    {
        $this->ci->db->where('id', $district_id);
        $this->ci->db->delete(DISTRICTS_TABLE);
    }

    /**
     * Return districts objects as array
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     results per page
     * @param array   $order_by          sorting data
     * @param array   $params            filters parameters
     * @param array   $filter_object_ids filters identifiers
     * @param integer $lang_id           language identifier
     *
     * @return array
     */
    public function get_districts($page = null, $items_on_page = null, $order_by = array(), $params = array(), $filter_object_ids = array(), $lang_id = false)
    {
        if ($lang_id === false) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        $select_attrs = $this->attrs_district;
        if ($lang_id) {
            $select_attrs = array_diff($select_attrs, array('name'));
            $select_attrs[] = 'lang_' . $lang_id . ' as name';
        }

        $this->ci->db->select(implode(", ", $select_attrs))->from(DISTRICTS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            $all_fields = $this->attrs_district;
            foreach ($this->ci->pg_language->languages as $lang_id => $lang_data) {
                $all_fields[] = 'lang_' . $lang_id;
            }
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $all_fields)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }
        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = array();
            foreach ($results as $r) {
                $this->district_cache[$r['id']] = $data[$r['id']] = $r;
            }

            return $data;
        }

        return array();
    }

    /**
     * Return districts objects as array
     *
     * @param array   $filter_object_ids filters identifiers
     * @param integer $lang_id           language identifier
     *
     * @return array
     */
    public function get_districts_by_id($filter_object_ids = array(), $lang_id = false)
    {
        if ($lang_id === false) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        $select_attrs = $this->attrs_district;
        if ($lang_id) {
            $select_attrs = array_diff($select_attrs, array('name'));
            $select_attrs[] = 'lang_' . $lang_id . ' as name';
        }

        if (!isset($filter_object_ids) || !is_array($filter_object_ids)) {
            return array();
        }

        $data = array();

        foreach ($filter_object_ids as $index => $id) {
            if (isset($this->district_cache[$id])) {
                $data[$id] = $this->district_cache[$id];
                unset($filter_object_ids[$index]);
            }
        }

        if (empty($filter_object_ids)) {
            return $data;
        }

        $this->ci->db->select(implode(", ", $select_attrs))->from(DISTRICTS_TABLE);

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $this->district_cache[$r['id']] = $data[$r['id']] = $r;
            }
        }

        return $data;
    }

    /**
     * Return districts objects by radius
     *
     * @param float   $lat               point latitude
     * @param float   $lon               point longitude
     * @param integer $radius            search radius
     * @param string  $radius_type       radius measurement
     * @param integer $page              page of results
     * @param integer $items_on_page     results per page
     * @param array   $params            filters parameters
     * @param array   $filter_object_ids filters identifiers
     *
     * @return array
     */
    public function get_districts_by_radius($lat, $lon, $radius = 10, $radius_type = "km", $page = null, $items_on_page = null, $params = array(), $filter_object_ids = array())
    {
        $lang_id = $this->ci->pg_language->current_lang_id;

        $select_attrs = $this->attrs_district;
        $select_attrs = array_diff($select_attrs, array('name'));
        $select_attrs[] = 'lang_' . $lang_id . ' as name';

        $this->ci->db->select(implode(", ", $select_attrs))->from(DISTRICTS_TABLE);

        $radius = ($radius_type == "mile") ? $radius : $radius * 0.6213712;
        $this->ci->db->where('(POW((69.1*(lon-"' . $lon . '")*cos(' . $lat . '/57.3)),"2")+POW((69.1*(lat-"' . $lat . '")),"2"))<(' . ($radius * $radius) . ')');

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = array();
            foreach ($results as $r) {
                $this->district_cache[$r['id']] = $data[$r['id']] = $r;
            }

            return $data;
        }

        return array();
    }

    /**
     * Return number of districts
     *
     * @param array $params            filters parameters
     * @param array $filter_object_ids filters identifiers
     *
     * @return integer
     */
    public function get_districts_count($params = array(), $filter_object_ids = array())
    {
        $this->ci->db->select("COUNT(*) AS cnt")->from(DISTRICTS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Return district object by identifier
     *
     * @param integer $district_id district identifier
     * @param integer $lang_id     language identifier
     * @param array   $languages   languages data
     *
     * @return array
     */
    public function get_district($district_id, $lang_id = false, $languages = array())
    {
        $cache_enabled = false;

        if ($lang_id === false) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        $select_attrs = $this->attrs_district;
        if ($lang_id) {
            $select_attrs = array_diff($select_attrs, array('name'));
            $select_attrs[] = 'lang_' . $lang_id . ' as name';
            $cache_enabled = true;
        }

        if (is_array($languages) && count($languages) > 0) {
            foreach ($languages as $id => $value) {
                $select_attrs[] = 'lang_' . $id . ' as lang_' . $id;
            }
            $cache_enabled = false;
        }

        if ($cache_enabled && isset($this->district_cache[$district_id])) {
            return $this->district_cache[$district_id];
        }

        $this->ci->db->select(implode(", ", $select_attrs))->from(DISTRICTS_TABLE)->where("id", $district_id);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            if ($cache_enabled) {
                $this->district_cache[$district_id] = $results[0];
            }

            return $results[0];
        }

        return array();
    }

    /**
     * Return district object by name
     *
     * @param string  $district_name district name
     * @param integer $city_id       city identifier
     * @param integer $region_id     region identifier
     * @param string  $country_id    country code
     * @param integer $lang_id       language identifier
     *
     * @return array
     */
    public function get_district_by_name($district_name, $city_id, $region_id = 0, $country_id = "", $lang_id = false)
    {
        if (!$lang_id) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }
        $this->ci->db->select(implode(", ", $this->attrs_city))->from(DISTRICTS_TABLE)->where('lang_' . $lang_id, $city_name);
        if ($city_id) {
            $this->ci->db->where("id_city", $city_id);
        }
        if ($region_id) {
            $this->ci->db->where("id_region", $region_id);
        }
        if ($country_id) {
            $this->ci->db->where("country_code", $country_id);
        }
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $results[0];
        }

        return array();
    }

    /**
     * Save district object to data source
     *
     * @param integer $district_id district identifier
     * @param array   $data        district data
     * @param array   $langs       languages data
     *
     * @return integer
     */
    public function save_district($district_id, $data, $langs = array())
    {
        if (is_array($langs) && count($langs) > 0) {
            foreach ($langs as $id => $value) {
                $data['lang_' . $id] = $value;
            }
        }
        if (empty($district_id)) {
            $this->ci->db->insert(DISTRICTS_TABLE, $data);
            $district_id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $district_id);
            $this->ci->db->update(DISTRICTS_TABLE, $data);
        }

        return $district_id;
    }

    /**
     * Save district object priority
     *
     * @param string  $district_id district identifier
     * @param integer $priority    priority value
     *
     * @return void
     */
    public function set_district_priority($district_id, $priority)
    {
        $data["priority"] = intval($priority);
        $data["sorted"] = 1;
        $this->ci->db->where("id", $district_id);
        $this->ci->db->update(DISTRICTS_TABLE, $data);
    }

    public function install_country($country_code)
    {
        $country_data = $this->get_cache_country_by_code($country_code);
        if (empty($country_data)) {
            return false;
        }
        $insert_data = array(
            'code' => $country_code,
            'languages' => $country_code,
            'name' => $country_data["name"],
            'areainsqkm' => $country_data["areainsqkm"],
            'continent' => $country_data["continent"],
            'currency' => $country_data["currency"],
            'priority' => $this->get_country_max_priority() + 1,
        );

        foreach ($this->pg_language->languages as $id => $value) {
            $insert_data['lang_' . $id] = $country_data['name'];
        }

        $this->save_country($country_code, $insert_data, "add");
    }

}
