<?php

/**
 * Export module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

define("EXPORT_DRIVERS_TABLE", DB_PREFIX . "export_drivers");

/**
 * Export driver model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Export
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Export_driver_model extends Model
{

    /**
     * Link to CodeIgniter model
     *
     * @var object
     */
    protected $CI;

    /**
     * Link to database object
     *
     * @var object
     */
    protected $DB;

    /**
     * Properties of export object in data source
     *
     * @var array
     */
    protected $_fields = array(
        "id",
        "gid",
        "link",
        "settings",
        "elements",
        "date_created",
        "date_modified",
        "editable",
        "status",
    );

    /**
     * Driver objects cache
     *
     * @var array
     */
    protected $driver_cache = array();

    /**
     * Available field types
     *
     * @var array
     */
    public $field_types = array("text", "int", "file", "url");

    /**
     * Constructor
     *
     * @return Export_driver_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
        $this->DB = &$this->CI->db;
    }

    /**
     * Return driver object by guid
     *
     * @param string  $driver_gid driver guid
     * @param boolean $formatted  format results
     *
     * @return array
     */
    public function get_driver_by_gid($driver_gid, $formatted = true)
    {
        if (empty($this->driver_cache[$driver_gid])) {
            $this->DB->select(implode(", ", $this->_fields))->from(EXPORT_DRIVERS_TABLE)->where("gid", $driver_gid);
            $results = $this->DB->get()->result_array();
            if (!empty($results) && is_array($results)) {
                if ($formatted) {
                    $this->driver_cache[$driver_gid] = $this->format_driver($results[0]);
                } else {
                    return $results[0];
                }
            } else {
                return array();
            }
        }

        return $this->driver_cache[$driver_gid];
    }

    /**
     * Return export driver objects as array
     *
     * @param boolean $formatted format results
     *
     * @return array
     */
    public function get_drivers($formatted = true)
    {
        $data = array();
        $this->DB->select(implode(", ", $this->_fields))->from(EXPORT_DRIVERS_TABLE);
        $results = $this->DB->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $result) {
                if ($formatted) {
                    $data[] = $this->format_driver($result);
                } else {
                    $data[] = $result;
                }
            }
        }

        return $data;
    }

    /**
     * Save driver object to data source
     *
     * @param string $driver_gid driver GUID
     * @param array  $data       driver data
     *
     * @return void
     */
    public function save_driver($driver_gid, $data)
    {
        if (!$driver_gid) {
            $data["date_created"] = date("Y-m-d H:i:s");
            $data["date_modified"] = date("Y-m-d H:i:s");
            $this->DB->insert(EXPORT_DRIVERS_TABLE, $data);
            $this->DB->insert_id();
        } else {
            $this->DB->where("gid", $driver_gid);
            $this->DB->update(EXPORT_DRIVERS_TABLE, $data);
        }
    }

    /**
     * Remove driver object by guid
     *
     * @param string $driver_gid driver guid
     *
     * @return void
     */
    public function remove_driver($driver_gid)
    {
        $this->DB->where("gid", $driver_gid);
        $this->DB->delete(EXPORT_DRIVERS_TABLE);
    }

    /**
     * Activate driver object by guid
     *
     * @param string $driver_gid driver guid
     *
     * @return void
     */
    public function activate_driver($driver_gid)
    {
        $data["status"] = "1";
        $this->save_driver($driver_gid, $data);
    }

    /**
     * De-activate driver object by guid
     *
     * @param string $driver_gid driver guid
     *
     * @return void
     */
    public function deactivate_driver($driver_gid)
    {
        $data["status"] = "0";
        $this->save_driver($driver_gid, $data);
    }

    /**
     * Validate driver object for saving to data source
     *
     * @param string $driver_gid driver guid
     * @param array  $data       driver data
     *
     * @return array
     */
    public function validate_driver($driver_gid, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data['id'])) {
            $return['data']['id'] = intval($data['id']);
            if (empty($return['data']['id'])) {
                unset($return['data']['id']);
            }
        }

        if (isset($data['gid'])) {
            $return['data']['gid'] = trim(strip_tags($data["gid"]));
            if (empty($return['data']['gid'])) {
                $return["errors"][] = l("error_empty_gid", "export");
            }
        } elseif (!$driver_gid) {
            $return["errors"][] = l("error_empty_gid", "export");
        }

        if (isset($data['link'])) {
            $return['data']['link'] = trim(strip_tags($data["link"]));
            if (empty($return['data']['link'])) {
                unset($return['data']['link']);
            }
        }

        if (isset($data["settings"])) {
            if (!is_array($data["settings"])) {
                $data["settings"] = array();
            }
            $return["data"]["settings"] = serialize($data["settings"]);
        }

        if (isset($data["elements"])) {
            if (!is_array($data["elements"])) {
                $data["elements"] = array();
            }
            $return["data"]["elements"] = serialize($data["elements"]);
            foreach ($data['elements'] as $field_data) {
                if (empty($field_data['name'])) {
                    $return["errors"][] = l("error_empty_field_name", "export");
                }
                if (empty($field_data['type'])) {
                    $return["errors"][] = l("error_empty_field_type", "export");
                } elseif (!in_array($field_data['type'], $this->field_types)) {
                    $return["errors"][] = l("error_invalid_field_type", "export");
                }
            }
        }

        if (isset($data['date_created'])) {
            $value = strtotime($data['date_created']);
            if ($value > 0) {
                $return['data']['date_created'] = date("Y-m-d", $value);
            }
        }

        if (isset($data['date_modified'])) {
            $value = strtotime($data['date_modified']);
            if ($value > 0) {
                $return['data']['date_modified'] = date("Y-m-d", $value);
            }
        }

        if (isset($data["editable"])) {
            $return["data"]["editable"] = intval($data["editable"]);
        }

        if (isset($data["status"])) {
            $return["data"]["status"] = intval($data["status"]);
        }

        return $return;
    }

    /**
     * Format driver object
     *
     * @param array $data driver data
     *
     * @return array
     */
    public function format_driver($data)
    {
        $data["output_name"] = l("driver_name_" . $data["gid"], "export");
        $data["settings"] = is_string($data["settings"]) ? (array) unserialize($data["settings"]) : array();
        $data["elements"] = is_string($data["elements"]) ? (array) unserialize($data["elements"]) : array();

        return $data;
    }

    /**
     * Return driver format by default
     *
     * @param array $driver_gid driver guid
     *
     * @return array
     */
    public function format_default_driver($driver_gid)
    {
        return;
    }

    /**
     * Return filename of export data by default
     *
     * @return string
     */
    public function get_filename()
    {
        return "export.txt";
    }

    /**
     * Generate output file
     *
     * @param string $filename output filename
     * @param array  $data     export data
     * @param array  $settings driver settings
     *
     * @return void
     */
    public function generate($filename, $data, $settings)
    {
        
    }

    /**
     * Load to provider data service
     *
     * @param string $driver_gid driver GUID
     *
     * @return array
     */
    public function loadDrivers($driver_gid, $format = false)
    {
        $model_name = "export_" . $driver_gid . "_model";
        $this->CI->load->model("export/models/drivers/" . $model_name, $model_name);

        $return = $this->CI->{$model_name}->drivers;
        if ($format) {
            return $this->format_driver($return);
        }

        return $return;
    }

    /**
     * Load driver demo data
     *
     * @param string $driver_gid driver GUID
     *
     * @return array
     */
    public function loadDriverDemoData($driver_gid)
    {
        $model_name = "export_" . $driver_gid . "_model";
        $this->CI->load->model("export/models/drivers/" . $model_name, $model_name);
        $this->CI->load->model("export/models/Export_module_model");

        $return = $this->CI->{$model_name}->demo_data;
        if (isset($return['module'])) {
            $return['id_object'] = $this->CI->Export_module_model->get_modules(false, ['module' => $return['module']])[0]['id'];
            unset($return['module']);
        }
        if (!empty($return)) {
            return $return;
        }

        return false;
    }

}
