<?php

/**
 * Import module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

define("IMPORT_DRIVERS_TABLE", DB_PREFIX . "import_drivers");

/**
 * Import driver model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Import
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Import_driver_model extends Model
{
    /**
     * Link to CodeIgniter model
     *
     * @var object
     */
    protected $CI;

    /**
     * Link to database object
     *
     * @var object
     */
    protected $DB;

    /**
     * Properties of import driver object in data source
     *
     * @var array
     */
    protected $_fields = array(
        "id",
        "gid",
        "link",
        "settings",
        "elements",
        "editable",
        "is_upload",
        "is_integration",
        "is_logging",
        "status",
        "date_created",
        "date_modified",
    );

    /**
     * Drivers cache
     *
     * @var array
     */
    protected $driver_cache = array();

    /**
     * Available field types
     *
     * @var array
     */
    public $field_types = array("text", "int", "file", "url");

    /**
     * Driver settings
     *
     * @var array
     */
    public $settings = array();

    /**
     * Constructor
     *
     * @return Import_driver_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
        $this->DB = &$this->CI->db;
    }

    /**
     * Return driver object by guid
     *
     * @param string  $gid       driver guid
     * @param boolean $formatted format results
     *
     * @return array
     */
    public function get_driver_by_gid($gid, $formatted = true)
    {
        if (empty($this->driver_cache[$gid])) {
            $this->DB->select(implode(", ", $this->_fields))->from(IMPORT_DRIVERS_TABLE)->where("gid", $gid);
            $results = $this->DB->get()->result_array();
            if (!empty($results) && is_array($results)) {
                if ($formatted) {
                    $this->driver_cache[$gid] = $this->format_driver($results[0]);
                } else {
                    return $results[0];
                }
            } else {
                return array();
            }
        }

        return $this->driver_cache[$gid];
    }

    /**
     * Return driver objects as array
     *
     * @param boolean $formatted format results
     *
     * @return array
     */
    public function get_drivers($formatted = true)
    {
        $data = array();
        $this->DB->select(implode(", ", $this->_fields))->from(IMPORT_DRIVERS_TABLE);
        $results = $this->DB->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $result) {
                if ($formatted) {
                    $data[] = $this->format_driver($result);
                } else {
                    $data[] = $result;
                }
            }
        }

        return $data;
    }

    /**
     *  Load driver
     *
     *  @return void
     */
    public function get_driver_data()
    {
        return $this->driver->driver_data;
    }

    /**
     * Save driver object to data source
     *
     * @param string $gid  driver guid
     * @param array  $data driver data
     *
     * @return void
     */
    public function save_driver($gid, $data)
    {
        if (!$gid) {
            $data["date_created"] = date("Y-m-d H:i:s");
            $data["date_modified"] = date("Y-m-d H:i:s");
            $this->DB->insert(IMPORT_DRIVERS_TABLE, $data);
        } else {
            $this->DB->where("gid", $gid);
            $this->DB->update(IMPORT_DRIVERS_TABLE, $data);
        }
    }

    /**
     * Remove driver object from data source
     *
     * @param string $driver_gid driver guid
     *
     * @return void
     */
    public function remove_driver($driver_gid)
    {
        $this->DB->where("gid", $driver_gid);
        $this->DB->delete(IMPORT_DRIVERS_TABLE);
    }

    /**
     * Activate driver object
     *
     * @param string $driver_gid driver guid
     *
     * @return void
     */
    public function activate_driver($driver_gid)
    {
        $data["status"] = "1";
        $this->save_driver($driver_gid, $data);
    }

    /**
     * Deactivate driver
     *
     * @param string $driver_gid driver guid
     *
     * @return void
     */
    public function deactivate_driver($driver_gid)
    {
        $data["status"] = "0";
        $this->save_driver($driver_gid, $data);
    }

    /**
     * Validate driver data
     *
     * @param string $gid  driver guid
     * @param array  $data data for validation
     *
     * @return array
     */
    public function validate_driver($gid, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data['id'])) {
            $return['data']['id'] = intval($data['id']);
            if (empty($return['data']['id'])) {
                unset($return['data']['id']);
            }
        }

        if (isset($data['gid'])) {
            $return['data']['gid'] = trim(strip_tags($data["gid"]));
            if (empty($return['data']['gid'])) {
                $return["errors"][] = l("error_empty_gid", "export");
            }
        } elseif (!$gid) {
            $return["errors"][] = l("error_empty_gid", "export");
        }

        if (isset($data['link'])) {
            $return['data']['link'] = trim(strip_tags($data["link"]));
            if (empty($return['data']['link'])) {
                unset($return['data']['link']);
            }
        }

        if (isset($data['settings'])) {
            $model_name = "Import_" . $gid . "_model";
            $this->CI->load->model("import/models/drivers/" . $model_name, $model_name);

            $validate_settings = $this->CI->{$model_name}->validate_settings($data['settings']);

            $return['errors'] = array_merge($return['errors'], $validate_settings['errors']);
            $return['data']['settings'] = serialize($validate_settings['data']);
        }

        if (isset($data['elements'])) {
            if (is_array($validate_settings['elements'])) {
                $return['data']['elements'] = serialize($data['elements']);
            } else {
                $return['data']['elements'] = '';
            }
        }

        if (isset($data["editable"])) {
            $return["data"]["editable"] = intval($data["editable"]);
        }

        if (isset($data["is_upload"])) {
            $return["data"]["is_upload"] = intval($data["is_upload"]);
        }

        if (isset($data["is_integration"])) {
            $return["data"]["is_integration"] = intval($data["is_integration"]);
        }

        if (isset($data["is_logging"])) {
            $return["data"]["is_logging"] = intval($data["is_logging"]);
        }

        if (isset($data["status"])) {
            $return["data"]["status"] = intval($data["status"]);
        }

        if (isset($data['date_created'])) {
            $value = strtotime($data['date_created']);
            if ($value > 0) {
                $return['data']['date_created'] = date("Y-m-d", $value);
            }
        }

        if (isset($data['date_modified'])) {
            $value = strtotime($data['date_modified']);
            if ($value > 0) {
                $return['data']['date_modified'] = date("Y-m-d", $value);
            }
        }

        return $return;
    }

    /**
     * Format driver object
     *
     * @param array $data driver data
     *
     * @return array
     */
    public function format_driver($data)
    {
        $data["output_name"] = l("driver_name_" . $data["gid"], "import");

        if (!empty($data['settings'])) {
            $data['settings'] = (array) unserialize($data['settings']);
        } else {
            $data['settings'] = array();
        }

        if (!empty($data['elements'])) {
            $data['elements'] = (array) unserialize($data['elements']);
        } else {
            $data['elements'] = array();
        }

        $data['input_types'] = array();

        if ($data['is_upload']) {
            $data['input_types'][] = 'upload';
        }
        if ($data['is_integration']) {
            $data['input_types'][] = 'integration';
        }

        return $data;
    }

    /**
     * Return default driver object
     *
     * @param string $driver_gid driver guid
     *
     * @return array
     */
    public function format_default_driver($driver_gid)
    {
        return;
    }

    /**
     * Parse uploaded file
     *
     * @param string  $filename  import filename
     * @param array   $settings  driver settings
     * @param array   $relations field relations
     * @param integer $position  start position
     *
     * @return array
     */
    public function parse($filename, $settings, $relations, $position)
    {
    }

    /**
     * Set element depth for repeating request
     *
     * @return void
     */
    public function set_depth()
    {
    }

    /**
     * Connect to provider data service
     *
     * @param integer $import_id import identifier
     *
     * @return array
     */
    public function connect($driver_gid)
    {
        $return = array("errors" => array(), "data" => array());

        $driver_data = $this->get_driver_by_gid($driver_gid, true);
        $model_name = "import_" . $driver_data["gid"] . "_model";
        $this->CI->load->model("import/models/drivers/" . $model_name, $model_name);

        $return = $this->CI->{$model_name}->connect($driver_data["settings"]);

        return $return;
    }

    /**
     * Load to provider data service
     *
     * @param string $driver_gid driver GUID
     *
     * @return array
     */
    public function loadDrivers($driver_gid, $format = false)
    {
        $model_name = "import_" . $driver_gid . "_model";
        $this->CI->load->model("import/models/drivers/" . $model_name, $model_name);

        $return = $this->CI->{$model_name}->drivers;
        if ($format) {
            return $this->format_driver($return);
        }

        return $return;
    }
}
