<?php

namespace Pg\Modules\Listings\Models;

/**
 * Listings module
 *
 * @package 	PG_RealEstate
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
define('LISTINGS_SEARCHES_TABLE', DB_PREFIX . 'listings_searches');

/**
 * Listings saved searches model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Listings
 * @category	models
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Listings_search_model extends \Model
{

    /**
     * Link to CodeIgniter object
     * 
     * @var object
     */
    protected $ci;

    /**
     * Properties of search object in data source
     * 
     * @var array
     */
    protected $_fields = array(
        'id',
        'id_user',
        'search_data',
        'date_search',
    );

    /**
     * Parameters of saved search
     * 
     * @var array
     */
    protected $saved_params = array(
        'keyword',
        'type',
        'id_country',
        'id_region',
        'id_city',
        'id_district',
        'id_category',
        'property_type',
        'price_min',
        'price_max',
        'square_min',
        'square_max',
        'field_editor_data',
        'with_photo',
        'by_open_house',
        'by_private',
        'by_discount',
        'id',
        'zip',
        'radius',
        'booking_date_start',
        'booking_date_end',
        'booking_guests',
    );

    /**
     * Separator of search criteria
     * 
     * @var string
     */
    protected $_separator = '; ';

    /**
     * Constructor
     * 
     * @return Listings_search_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return data of saved search by identifier
     * 
     * @param integer $search_id saved search identifier
     * @return array
     */
    public function getSearchById($search_id)
    {
        $this->ci->db->select(implode(', ', $this->_fields));
        $this->ci->db->from(LISTINGS_SEARCHES_TABLE);
        $this->ci->db->where('id', $search_id);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = $this->formatSearch($results[0]);
            return $data;
        }
    }

    /**
     * Filter saved searches by user
     * 
     * @param integer $user_id user identifier
     * @return array
     */
    protected function getSearchesByUser($user_id)
    {
        $params = array();
        if (!$user_id) {
            return array();
        }
        $params['where']['id_user'] = $user_id;
        return $params;
    }

    /**
     * Return saved search objects from data source as array
     * 
     * @param integer $page page of results
     * @param integer $items_on_page items per page
     * @param array $order_by sorting data
     * @param array $params sql criteria
     * @return array
     */
    protected function _getSearchesList($page = null, $items_on_page = null, $order_by = array(), $params = array())
    {
        $this->ci->db->from(LISTINGS_SEARCHES_TABLE)
                ->select('id, id_user, search_data, date_search');

        if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
            foreach ($params['where'] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params['where_in']) && is_array($params['where_in']) && count($params['where_in'])) {
            foreach ($params['where_in'] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
            foreach ($params['where_sql'] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                $this->ci->db->order_by($field . ' ' . $dir);
            }
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $this->formatSearches($results);
        }
        return array();
    }

    /**
     * Return number of saved search objects in data source
     * 
     * @param array $params sql criteria
     * @return integer
     */
    protected function _getSearchesCount($params)
    {
        $this->ci->db->select('COUNT(*) AS cnt')
                ->from(LISTINGS_SEARCHES_TABLE);

        if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
            foreach ($params['where'] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params['where_in']) && is_array($params['where_in']) && count($params['where_in'])) {
            foreach ($params['where_in'] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
            foreach ($params['where_sql'] as $value) {
                $this->ci->db->where($value);
            }
        }

        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            return intval($result[0]->cnt);
        } else {
            return 0;
        }
    }

    /**
     * Return saved search objects as array
     * 
     * @param array $filters filters data
     * @param integer $page page of results
     * @param integer $items_on_page items per page
     * @param array $order_by sorting data
     * @return array
     */
    public function getSearchesList($filters = array(), $page = null, $items_on_page = null, $order_by = array())
    {
        $params = array();
        foreach ($filters as $filter => $value) {
            $params = array_merge($params, $this->{'getSearchesBy' . ucFirst($filter)}($value));
        }
        return $this->_getSearchesList($page, $items_on_page, $order_by, $params);
    }

    /**
     * Return number of saved search objects
     * 
     * @param array $filters filters data
     * @return integer
     */
    public function getSearchesCount($filters = array())
    {
        $params = array();
        foreach ($filters as $filter => $value) {
            $params = array_merge($params, $this->{'getSearchesBy' . ucFirst($filter)}($value));
        }
        return $this->_getSearchesCount($params);
    }

    /**
     * Format data of saved search object
     * 
     * @param array $data saved search data
     * @return array
     */
    public function formatSearch($data)
    {
        return array_shift($this->formatSearches(array($data)));
    }

    /**
     * Format data of saved searches objects
     * 
     * @param array $data data of saved searches objects
     * @return array
     */
    public function formatSearches($data)
    {
        $this->ci->load->model('Listings_model');

        $operation_types = $this->ci->Listings_model->get_operation_types(true);

        foreach ($data as $key => $search) {
            $search_data = $search['search_data'] = $search['search_data'] ? (array) unserialize($search['search_data']) : array();
            $current_search_data = array();
            if (isset($search_data['type']) && in_array($search_data['type'], $operation_types)) {
                $operation_type = $search['search_data']['type'];
            } else {
                $operation_type = $this->ci->Listings_model->get_operation_type_gid_default(true);
            }

            if ($operation_type == 'sold') {
                $search['name'] = l('text_sold', 'listings');
            } else {
                $search['name'] = ld_option('operation_type_search', 'listings', $operation_type);
            }

            $exclude = array('type');

            if (isset($search_data['id_category'])) {
                $this->ci->load->helper('properties');
                if (isset($search_data['property_type'])) {
                    $value = property_value($search_data);
                    $search['name'] .= $this->_separator . ($value ? $value : $search_data['property_type']);
                    $exclude[] = 'property_type';
                } else {
                    $value = category_value($search_data);
                    $search['name'] .= $this->_separator . ($value ? $value : $search_data['id_category']);
                }
                $exclude[] = 'id_category';
            }

            if (!empty($search_data['id_country'])) {
                $this->ci->load->helper('countries');
                $location_data = array();
                $location_data['country'] = $search_data['id_country'];
                $exclude[] = 'id_country';

                if (!empty($search_data['id_region'])) {
                    $location_data['region'] = $search_data['id_region'];
                    $exclude[] = 'id_region';
                    if (!empty($search_data['id_city'])) {
                        $location_data['city'] = $search_data['id_city'];
                        $exclude[] = 'id_city';
                        if (!empty($search_data['id_district'])) {
                            $location_data['district'] = $search_data['id_district'];
                            $exclude[] = 'id_district';
                            $location = districts_output_format(array($location_data));
                            if (!empty($location)) {
                                $search['name'] .= $this->_separator . $location[0];
                            }
                        } else {
                            $location = cities_output_format(array($location_data));
                            if (!empty($location)) {
                                $search['name'] .= $this->_separator . $location[0];
                            }
                        }
                    } else {
                        $location = regions_output_format(array($location_data));
                        if (!empty($location)) {
                            $search['name'] .= $this->_separator . $location[0];
                        }
                    }
                } else {
                    $location = countries_output_format(array($location_data));
                    if (!empty($location)) {
                        $search['name'] .= $this->_separator . $location[0];
                    }
                }
            }

            $date_format = $this->ci->pg_date->get_format('date_literal', 'date');

            if ($this->ci->pg_module->is_module_active('listings_booking')) {
                if (isset($search_data['booking_date_start'])) {
                    $search_data['booking_date_start'] = date($date_format, strtotime($search_data['booking_date_start']));
                }

                if (isset($search_data['booking_date_end'])) {
                    $search_data['booking_date_end'] = date($date_format, strtotime($search_data['booking_date_end']));
                }
            }

            if (!empty($search_data['by_agents'])) {
                $search_data['by_agents'] = l('yes_str', 'start');
            }

            if (!empty($search_data['by_private'])) {
                $search_data['by_private'] = l('yes_str', 'start');
            }

            if (!empty($search_data['by_open_house'])) {
                $search_data['by_open_house'] = l('yes_str', 'start');
            }

            if (!empty($search_data['with_photo'])) {
                $search_data['with_photo'] = l('yes_str', 'start');
            }

            $this->load->helper('start');

            // price
            $price = '';
            if (isset($search_data['price_min'])) {
                $price = l('text_price_from', 'listings') . ' ' . strip_tags(currency_format_output(array('value' => $search_data['price_min'])));
                $exclude[] = 'price_min';
            }

            if (isset($search_data['price_max'])) {
                if ($price) {
                    $price .= ' ';
                }
                $price .= l('text_price_to', 'listings') . ' ' . strip_tags(currency_format_output(array('value' => $search_data['price_max'])));
                $exclude[] = 'price_max';
            }

            if (!empty($price)) {
                $search['name'] .= $this->_separator . '<span dir="ltr">' . $price . '</span>';
            }

            // square
            $square = '';
            if (isset($search_data['square_min'])) {
                $square = l('text_price_from', 'listings') . ' ' . $search_data['square_min'];
                $exclude[] = 'square_min';
            }

            if (isset($search_data['square_max'])) {
                if ($square) {
                    $square .= ' ';
                }
                $square .= l('text_price_to', 'listings') . ' ' . $search_data['square_max'];
                $exclude[] = 'square_max';
            }

            if (!empty($square)) {
                $search['name'] .= $this->_separator . l('field_square', 'listings') . ': ' . $square;
            }

            if (isset($search_data['field_editor_data'])) {
                $property_type_gid = $this->ci->Listings_model->get_field_editor_type($search_data);

                $this->ci->load->model('Field_editor_model');
                $this->ci->Field_editor_model->initialize($property_type_gid);

                $fields_gids = array();
                foreach ($search_data['field_editor_data'] as $form_name => $form_data) {
                    $fields_gids = array_merge($fields_gids, array_keys($form_data));
                }

                $params = array('where_in' => array('gid' => array_keys($fields_gids)));
                $fields_data = $this->ci->Field_editor_model->get_fields_list($params);

                $field_editor_data = array('gids' => array(), 'data' => array());
                $field_editor_range = array('gids' => array(), 'data' => array());
                foreach ($search_data['field_editor_data'] as $form_name => $form_data) {
                    foreach ($form_data as $field_gid => $field_data) {
                        if (isset($field_data['range']) && is_array($field_data['range'])) {
                            if (isset($field_data['range']['min'])) {
                                $field_editor_data['gids'][$field_gid] = $field_data['range']['min'];
                                $field_editor_data['data'][$fields_data[$field_gid]['field_name']] = $field_data['range']['min'];
                            } else {
                                $field_editor_data['gids'][$field_gid] = 0;
                                $field_editor_data['data'][$fields_data[$field_gid]['field_name']] = 0;
                            }
                            if (isset($field_data['range']['max'])) {
                                $field_editor_range['gids'][$field_gid] = $field_data['range']['max'];
                                $field_editor_range['data'][$fields_data[$field_gid]['field_name']] = $field_data['range']['max'];
                            } else {
                                $field_editor_range['gids'][$field_gid] = 0;
                                $field_editor_range['data'][$fields_data[$field_gid]['field_name']] = 0;
                            }
                        } else {
                            $field_editor_data['gids'][$field_gid] = $field_data;
                            $field_editor_data['data'][$fields_data[$field_gid]['field_name']] = $field_data;
                        }
                    }
                }

                $values = array();

                if (!empty($field_editor_data['gids'])) {
                    $params = array('where_in' => array('gid' => array_keys($field_editor_data['gids'])));
                    $results = $this->ci->Field_editor_model->format_list_fields_for_view($params, array($field_editor_data['data']));
                    foreach ($results as $result) {
                        foreach ($result as $field_gid => $field_data) {
                            if ($field_data['value_str']) {
                                $value = $field_data['value_str'];
                            } elseif ($field_data['value_dec']) {
                                $value = $field_data['value_dec'];
                            } elseif ($field_data['value_original']) {
                                $value = $field_data['value_original'];
                            } else {
                                $value = $field_data['value'];
                            }
                            $values[$field_gid] = $value || $field_editor_data['gids'][$field_gid] !== 0 ? $value : 0;
                        }
                    }
                }

                if (!empty($field_editor_range['gids'])) {
                    $params = array('where_in' => array('gid' => array_keys($field_editor_range['gids'])));
                    $results = $this->ci->Field_editor_model->format_list_fields_for_view($params, array($field_editor_range['data']));
                    foreach ($results as $result) {
                        foreach ($result as $field_gid => $field_data) {
                            if (!empty($field_data['value_str'])) {
                                $value = $field_data['value_str'];
                            } elseif (!empty($field_data['value_dec'])) {
                                $value = $field_data['value_dec'];
                            } elseif (!empty($field_data['value_original'])) {
                                $value = $field_data['value_original'];
                            } else {
                                $value = $field_data['value'];
                            }

                            if ($value) {
                                if (isset($values[$field_gid]) && $values[$field_gid]) {
                                    $values[$field_gid] .= '-' . $value;
                                } else {
                                    $values[$field_gid] = '&le;' . $value;
                                }
                            } elseif (isset($values[$field_gid])) {
                                $values[$field_gid] = '&ge;' . $values[$field_gid];
                            }
                        }
                    }
                }

                foreach ($values as $field_gid => $value) {
                    $search['name'] .= $this->_separator . $fields_data[$field_gid]['name'] . ': ' . $value;
                }

                $exclude[] = 'field_editor_data';
            }

            foreach ($search_data as $k => $v) {
                if (in_array($k, $exclude)) {
                    continue;
                }
                $search['name'] .= $this->_separator . l('field_' . $k, 'listings') . ': ' . $v;
            }
            $data[$key] = $search;
        }
        return $data;
    }

    /**
     * Validate data of saved search for saving to data source
     * 
     * @param integer $search_id saved search identifier
     * @param array $data saved search data
     * @return array
     */
    public function validateSearch($search_id, $data, $check_uniqueness = false)
    {
        $return = array('errors' => array(), 'data' => array());

        if (isset($data['id_user'])) {
            $return['data']['id_user'] = intval($data['id_user']);
        }

        if (isset($data['search_data'])) {
            $search_data = array();
            $this->ci->load->model('Listings_model');

            foreach ($this->saved_params as $f) {
                if (!empty($data['search_data'][$f])) {
                    $search_data[$f] = $data['search_data'][$f];
                }
            }

            if (!empty($search_data)) {
                $return['data']['search_data'] = serialize($search_data);
            } else {
                $return['errors'][] = l('error_empty_save_search_data', 'listings');
            }
            if ($check_uniqueness && empty($return['errors']) && 
                    !$this->isSearchUnique($return['data']['search_data'])) {
                $return['errors'][] = l('error_search_data_not_unique', 'listings');
            }
        }
        return $return;
    }

    public function isSearchUnique($search_data)
    {
        return 0 === $this->_getSearchesCount(['where' => ['hash' => md5($search_data)]]);
    }

    /**
     * Save search object to data source
     * 
     * @param integer $search_id search identifier
     * @param array $data search data
     * @return integer
     */
    public function saveSearch($search_id, $data)
    {
        if (!empty($data['search_data'])) {
            $data['hash'] = md5($data['search_data']);
        }
        if (empty($search_id)) {
            $data['date_search'] = date('Y-m-d H:i:s');
            $this->ci->db->insert(LISTINGS_SEARCHES_TABLE, $data);
            $search_id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $search_id);
            $this->ci->db->update(LISTINGS_SEARCHES_TABLE, $data);
        }
        return $search_id;
    }

    /**
     * Remove search object from data source by identifier
     * 
     * @param integer $search_id search identifier
     * @retun void
     */
    public function deleteSearch($search_id)
    {
        $this->ci->db->where('id', $search_id);
        $this->ci->db->delete(LISTINGS_SEARCHES_TABLE);
    }

}
