<?php

namespace Pg\Modules\Moderation\Models;

/**
 * Moderation module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
define('MODERATION_ITEMS_TABLE', DB_PREFIX . 'moderation_items');

/**
 * Moderation Model
 *
 * модель никогда не меняет статуса уже активного элемента
 * каждый вызов add_moderation_item добавляет запись в соотв с типом мод.объекта
 *
 * @package 	PG_RealEstate
 * @subpackage 	Moderation
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Moderation_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Moderation types
     *
     * @var array
     */
    public $types;

    /**
     * Constructor
     *
     * @return Moderation_model
     */
    public function __construct()
    {
        parent::__construct();

        $this->ci = &get_instance();
        $this->ci->load->model('moderation/models/Moderation_type_model');
        $this->ci->load->model('moderation/models/Moderation_badwords_model');
    }

    /**
     * Return moderation type by GUID
     *
     * @param string $type_name type GUID
     *
     * @return array
     */
    public function get_moderation_type($type_name)
    {
        if (!isset($this->types[$type_name])) {
            $type_data = $this->ci->Moderation_type_model->get_type_by_name($type_name);
            if (!is_array($type_data) || !count($type_data)) {
                return false;
            }
            $this->types[$type_data["id"]] = $type_data;
            $this->types[$type_name] = $type_data;
        }

        return $this->types[$type_name];
    }

    /**
     * Return moderation type by identifier
     *
     * @param integer $type_id type identifier
     *
     * @return array
     */
    public function get_moderation_type_by_id($type_id)
    {
        if (!isset($this->types[$type_id])) {
            $type_data = $this->ci->Moderation_type_model->get_type_by_id($type_id);
            if (!is_array($type_data) || !count($type_data)) {
                return false;
            }
            $this->types[$type_id] = $type_data;
            $this->types[$type_data["name"]] = $type_data;
        }

        return $this->types[$type_id];
    }

    /**
     * Return status of moderation type
     *
     * 0 - without moderation
     * 1 - pre-moderation
     * 2 - post-moderation
     *
     * @param string $type_name type GUID
     *
     * @return integer
     */
    public function get_moderation_type_status($type_name)
    {
        $type_data = $this->get_moderation_type($type_name);
        switch ($type_data["mtype"]) {
            case "0": $status = 1;
                break;
            case "1": $status = 1;
                break;
            case "2": $status = 0;
                break;
        }

        return $status;
    }

    /**
     * Add object to moderation
     *
     * @param string  $type_name type GUID
     * @param integer $obj_id    object identifier
     *
     * @return void
     */
    public function add_moderation_item($type_name, $obj_id)
    {
        $type_data = $this->get_moderation_type($type_name);
        if ($type_data["mtype"] == 0) {
            return false;
        }
        $type_id = intval($type_data["id"]);

        /// если объект уже занесен то обновляем дату
        if ($this->isset_moderation_item($type_name, $obj_id)) {
            $attrs["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->where('id_type', $type_id);
            $this->ci->db->where('id_object', $obj_id);
            $this->ci->db->update(MODERATION_ITEMS_TABLE, $attrs);
        } else {
            $attrs["date_add"] = date("Y-m-d H:i:s");
            $attrs["id_type"] = $type_id;
            $attrs["id_object"] = $obj_id;
            $this->ci->db->insert(MODERATION_ITEMS_TABLE, $attrs);

            $this->ci->load->model('menu/models/Indicators_model');
            $this->ci->Indicators_model->add('new_item_for_moderation', $this->ci->db->insert_id());

            $send_admin_mail = $this->ci->pg_module->get_module_config("moderation", "moderation_send_mail");
            if ($send_admin_mail) {
                $emails = $this->ci->pg_module->get_module_config("moderation", "admin_moderation_emails");
                if ($emails) {
                    $mail_data = array('type' => l('mtype_' . $type_data['name'], 'moderation'));
                    $this->ci->load->model('Notifications_model');
                    $this->ci->Notifications_model->send_notification($emails, 'auser_need_moderate', $mail_data);
                }
            }
        }

        return;
    }

    /**
     * Check the object is moderated
     *
     * @param string  $type_name type GUID
     * @param integer $obj_id    object identifier
     *
     * @return boolean
     */
    public function isset_moderation_item($type_name, $obj_id)
    {
        $type_data = $this->get_moderation_type($type_name);
        $type_id = intval($type_data["id"]);

        $this->ci->db->select('COUNT(*) AS cnt')->from(MODERATION_ITEMS_TABLE)->where('id_type', $type_id)->where('id_object',
                                                                                                              $obj_id);
        $result = $this->ci->db->get()->result();
        if (!empty($result) && intval($result[0]->cnt)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Return moderation item by identifier
     *
     * @param integer $id item identifier
     *
     * @return array/false
     */
    public function get_moderation_item($id)
    {
        $id = intval($id);
        if (!$id) {
            return false;
        }
        $this->ci->db->select('id, id_type, id_object, date_add')->from(MODERATION_ITEMS_TABLE)->where("id", $id);
        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            return get_object_vars($result[0]);
        } else {
            return false;
        }
    }

    /**
     * Remove moderation item by identifier
     *
     * @param integer $id item identifier
     *
     * @return boolean
     */
    public function delete_moderation_item_by_id($id)
    {
        $id = intval($id);
        if (!$id) {
            return false;
        }
        $this->ci->db->delete(MODERATION_ITEMS_TABLE, array('id' => $id));

        $this->ci->load->model('menu/models/Indicators_model');
        $this->ci->Indicators_model->delete('new_item_for_moderation', $id, true);

        return;
    }

    /**
     * Remove moderations items by type identifier
     *
     * @param integer $type_id type identifier
     *
     * @return boolean
     */
    public function delete_moderation_items_by_type_id($type_id)
    {
        if (!intval($type_id)) {
            return false;
        }

        $this->ci->db->select('id')->from(MODERATION_ITEMS_TABLE)->where('id_type', $type_id);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $this->ci->db->where('id_type', $type_id);
            $this->ci->db->delete(MODERATION_ITEMS_TABLE);

            $this->ci->load->model('menu/models/Indicators_model');

            foreach ($results as $row) {
                $this->ci->Indicators_model->delete('new_item_for_moderation', $row['id'], true);
            }
        }

        return;
    }

    /**
     * Return moderation item by identifier
     *
     * @param string  $type_name type GUID
     * @param integer $obj_id    object identifier
     *
     * @return array/boolean
     */
    public function delete_moderation_item_by_obj($type_name, $obj_id)
    {
        $type_data = $this->get_moderation_type($type_name);
        $type_id = intval($type_data["id"]);

        if (is_array($obj_id) && count($obj_id)) {
            $obj_id_arr = $obj_id;
        } elseif (is_numeric($obj_id) && $obj_id > 0) {
            $obj_id_arr[] = intval($obj_id);
        } else {
            return false;
        }

        $this->ci->db->select('id')->from(MODERATION_ITEMS_TABLE)->where('id_type', $type_id)->where_in('id_object',
                                                                                                    $obj_id_arr);
        $results = $this->ci->db->get()->result_array();

        if (!empty($results) && is_array($results)) {
            $this->ci->db->where('id_type', $type_id);
            $this->ci->db->where_in('id_object', $obj_id_arr);
            $this->ci->db->delete(MODERATION_ITEMS_TABLE);

            $this->ci->load->model('menu/models/Indicators_model');

            foreach ($results as $row) {
                $this->ci->Indicators_model->delete('new_item_for_moderation', $row['id'], true);
            }
        }

        return;
    }

    /**
     * Return number of moderation object in data source
     *
     * @param string $type_name type GUID
     *
     * @return integer
     */
    public function get_moderation_list_count($type_name = "")
    {
        if ($type_name) {
            $type_data = $this->get_moderation_type($type_name);
            $this->ci->db->where('id_type', $type_data["id"]);
        }
        $this->ci->db->select('COUNT(*) AS cnt')->from(MODERATION_ITEMS_TABLE);
        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            return intval($result[0]->cnt);
        } else {
            return 0;
        }
    }

    /**
     * Return moderation objects as array
     *
     * @param string  $type_name     type GUID
     * @param integer $page          page of results
     * @param integer $list_per_page items per page
     * @param string  $parse_html    generate content of object
     *
     * @return array
     */
    public function get_moderation_list($type_name = "", $page = null, $list_per_page = null, $parse_html = true)
    {
        $this->ci->db->select('id, id_type, id_object, date_add')->from(MODERATION_ITEMS_TABLE);
        if ($type_name) {
            $type_data = $this->get_moderation_type($type_name);
            $this->ci->db->where('id_type', $type_data["id"]);
        }

        $this->ci->db->order_by("date_add DESC");
        if (!is_null($page) && !is_null($list_per_page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($list_per_page, $list_per_page * ($page - 1));
        }

        $result = $this->ci->db->get()->result();
        if (empty($result)) {
            return false;
        }

        foreach ($result as $item) {
            $type = $this->get_moderation_type_by_id($item->id_type);
            $item->type_name = $type["name"];
            $item->type = $type;
            $object_ids[$item->type_name][] = $item->id_object;
            if (strlen($type["view_link"])) {
                $item->view_link = site_url() . $type["view_link"] . $item->id_object;
            }
            if (strlen($type["edit_link"])) {
                $item->edit_link = site_url() . $type["edit_link"] . $item->id_object;
            }
            if (strlen($type["method_delete_object"])) {
                $item->avail_delete = true;
            }
            if (strlen($type["method_set_status"]) && intval($type["allow_to_decline"])) {
                $item->avail_decline = true;
            }
            $list[] = get_object_vars($item);
        }

        if ($parse_html && isset($object_ids) && is_array($object_ids)) {
            foreach ($object_ids as $type_name => $ids) {

                /// получем параметры типа
                $type = $this->types[$type_name];

                /// подключаем модель
                $model_name = ucfirst($type["model"]);
                $model_path = strtolower($type["module"] . "/models/") . $model_name;
                $this->ci->load->model($model_path);

                /// получаем данные обектов по ids (возвращается массив: id_object => object_data)
                $objects_data[$type_name] = $this->ci->$model_name->$type["method_get_list"]($ids);
            }

            foreach ($list as $key => $item) {
                if (isset($objects_data[$item["type_name"]][$item["id_object"]])) {
                    /// assign в шаблон, складываем html в переменную
                    $this->ci->view->assign('data', $objects_data[$item["type_name"]][$item["id_object"]]);
                    $list[$key]["html"] = $this->ci->view->fetch($item["type"]["template_list_row"], 'admin',
                                                                 $item["type"]["module"]);
                }
            }
        }

        return $list;
    }

    /**
     * Validate moderation settings
     *
     * @param array $data settings data
     *
     * @return array
     */
    public function validate_settings($data)
    {
        $return = array("errors" => array(), "data" => array());

        $return["data"]["moderation_send_mail"] = $data["moderation_send_mail"] ? 1 : 0;

        /// email
        if (isset($data['admin_moderation_emails'])) {
            $return["data"]["admin_moderation_emails"] = trim(strip_tags($data["admin_moderation_emails"]));

            if (!empty($return["data"]["admin_moderation_emails"])) {
                $this->ci->config->load("reg_exps", true);
                $email_expr = $this->ci->config->item("email", "reg_exps");
                $chunks = explode(',', $return["data"]["admin_moderation_emails"]);
                foreach ($chunks as $chunk) {
                    if (empty($chunk) || !preg_match($email_expr, trim($chunk))) {
                        $return["errors"][] = l("error_invalid_email", "moderation");
                        break;
                    }
                }
            } elseif ($return["data"]["moderation_send_mail"]) {
                $return["errors"][] = l("error_empty_email", "moderation");
            }
        }

        return $return;
    }

}
