<?php

namespace Pg\Modules\Payments\Models;

/**
 * Payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

define('PAYMENTS_SYSTEMS_TABLE', DB_PREFIX . 'payments_systems');
define('PAYMENTS_LOG_TABLE', DB_PREFIX . 'payments_log');

/**
 * Payment systems model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Payments
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Payment_systems_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Payment system object properties in data source
     *
     * @var array
     */
    protected $fields = array(
        'id',
        'gid',
        'name',
        'settings_data',
        'is_test',
        'in_use',
        'tarifs_type',
        'tarifs_editable',
        'tarifs_status',
        'tarifs_data',
        'disable',
        'date_add',
    );

    /**
     * Name of current driver object
     *
     * @var string
     */
    protected $current_driver_name = "";

    /**
     * Current driver object
     *
     * @var array
     */
    protected $driver;

    /**
     * Payment systems cache
     *
     * @var array
     */
    protected $systems_cache = array();

    /**
     * Use logging
     *
     * @var boolean
     */
    protected $log_data = true;

    /**
     * Payment sections
     *
     * @var array
     */
    protected $payment_sections = array('desktop' => 1, 'mobile' => 2, 'account' => 3, 'services' => 4);

    /**
     * Module guid
     *
     * @var array
     */
    protected $module_gid = 'payment_systems';

    /**
     * Constructor
     *
     * @return Payment_systems_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        foreach ($this->ci->pg_language->languages as $id => $value) {
            $this->fields[] = 'info_data_' . $value['id'];
        }
    }

    /**
     * Return payment system object by GUID
     *
     * @param string $gid system GUID
     *
     * @return array
     */
    public function get_system_by_gid($gid)
    {
        if (empty($this->systems_cache[$gid])) {
            $result = $this->ci->db->select(implode(", ", $this->fields))->from(PAYMENTS_SYSTEMS_TABLE)->where("gid", $gid)->get()->result_array();
            if (!empty($result)) {
                $this->systems_cache[$gid] = $this->format_system($result[0]);
            } else {
                $this->systems_cache[$gid] = array();
            }
        }

        return $this->systems_cache[$gid];
    }

    /**
     * Format payment system object
     *
     * @param array $data system data
     *
     * @return array
     */
    public function format_system($data)
    {
        $data["settings_data"] = unserialize($data["settings_data"]);
        $reference = $this->pg_language->ds->get_reference($this->module_gid, $data['gid'] . '_operators');
        $data["operators_data"] = isset($reference['option']) ? $reference['option'] : array();
        $data["tarifs_status"] = unserialize($data["tarifs_status"]);
        $data["tarifs_data"] = unserialize($data["tarifs_data"]);
        $data["info_data"] = $data["info_data_" . $this->pg_language->current_lang_id];
        $data['access'] = $this->get_system_access($data);

        return $data;
    }

    /**
     * Return payment systems objects as array
     *
     * @param array $params            sql criteria of query to data source
     * @param array $filter_object_ids filters identifiers
     * @param array $order_by          sorting data
     *
     * @return array
     */
    public function get_system_list($params = array(), $filter_object_ids = null, $order_by = null)
    {
        $this->ci->db->select(implode(", ", $this->fields));
        $this->ci->db->from(PAYMENTS_SYSTEMS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value, null, false);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = $this->systems_cache[$r["gid"]] = $this->format_system($r);
            }

            return $data;
        }

        return array();
    }

    /**
     * Return actived payment systems objects as array
     *
     * @param array $order_by sorting data
     * @param array $sections payment sections
     *
     * @return array
     */
    public function get_active_system_list($order_by = array("name" => "ASC"), $sections = array())
    {
        $params = array();
        if (!empty($sections)) {
            $disable_code = $this->get_disable_code($sections);
            $params['where_sql'][] = '' . $disable_code . '&~disable=' . $this->ci->db->escape($disable_code);
        }
        $params["where"]["in_use"] = 1;

        return $this->get_system_list($params, null, $order_by);
    }

    /**
     * Return number of payment systems objects
     *
     * @param array $params            sql criteria of query to data source
     * @param array $filter_object_ids filters identifiers
     *
     * @return integer
     */
    public function get_system_count($params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select("COUNT(*) AS cnt");
        $this->ci->db->from(PAYMENTS_SYSTEMS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Validate payment systems object for saving to data source
     *
     * @param integer $id   system identifier
     * @param array   $data system data
     *
     * @return array
     */
    public function validate_system($id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["gid"])) {
            $data["gid"] = strip_tags($data["gid"]);
            $data["gid"] = preg_replace("/[^a-z0-9]+/i", '', $data["gid"]);

            $return["data"]["gid"] = $data["gid"];

            if (empty($return["data"]["gid"])) {
                $return["errors"][] = l('error_system_gid_incorrect', 'payments');
            }

            $param["where"]["gid"] = $return["data"]["gid"];
            if (!empty($id)) {
                $param["where"]["id <>"] = $id;
            }
            $gid_count = $this->get_system_count($param);
            if ($gid_count > 0) {
                $return["errors"][] = l('error_system_gid_already_exists', 'payments');
            }
        }

        if (isset($data["name"])) {
            $return["data"]["name"] = strip_tags($data["name"]);
            if (empty($return["data"]["name"])) {
                $return["errors"][] = l('error_system_name_incorrect', 'payments');
            }
        }

        if (isset($data["settings_data"])) {
            $return["data"]["settings_data"] = serialize($data["settings_data"]);
        }

        if (isset($data["is_test"])) {
            $return["data"]["is_test"] = intval($data["is_test"]);
        }

        if (isset($data["in_use"])) {
            $return["data"]["in_use"] = intval($data["in_use"]);
        }

        if (isset($data["info_data"])) {
            $return['data'] = array_merge($return['data'], $data["info_data"]);
        }

        if (isset($data['tarifs_type'])) {
            $return["data"]["tarifs_type"] = intval($data["tarifs_type"]);
        }

        if (isset($data['tarifs_editable'])) {
            $return["data"]["tarifs_editable"] = $data["tarifs_editable"] ? 1 : 0;
        }

        if (isset($data['disable'])) {
            $return["data"]["disable"] = intval($data["disable"]);
        }

        if (isset($data["tarifs_status"])) {
            $return["data"]["tarifs_status"] = serialize($data["tarifs_status"]);
        }

        if (isset($data["tarifs_data"])) {
            $return["data"]["tarifs_data"] = serialize($data["tarifs_data"]);
        }

        return $return;
    }

    /**
     * Validate data of information fields
     *
     * @param array $data information data
     *
     * @return array
     */
    public function validate_info_data($data)
    {
        $return = array("errors" => array(), "data" => array());

        $default_lang_id = $this->ci->pg_language->current_lang_id;
        if (isset($data['info_data_' . $default_lang_id])) {
            $return['data']['info_data_' . $default_lang_id] = trim(strip_tags($data['info_data_' . $default_lang_id]));
            foreach ($this->pg_language->languages as $lid => $lang_data) {
                if ($lid == $default_lang_id) {
                    continue;
                }
                if (!isset($data['info_data_' . $lid]) || empty($data['info_data_' . $lid])) {
                    $return['data']['info_data_' . $lid] = $return['data']['info_data_' . $default_lang_id];
                } else {
                    $return['data']['info_data_' . $lid] = trim(strip_tags($data['info_data_' . $lid]));
                }
            }
        }

        return $return;
    }

    /**
     * Save payment system object to data source
     *
     * @param integer $id   system identifier
     * @param array   $data system data
     *
     * @return integer
     */
    public function save_system($id, $data)
    {
        if (is_null($id)) {
            $data["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(PAYMENTS_SYSTEMS_TABLE, $data);
            $id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $id);
            $this->ci->db->update(PAYMENTS_SYSTEMS_TABLE, $data);
        }
        unset($this->systems_cache[$data["gid"]]);

        return $id;
    }

    /**
     * Make payment system is used
     *
     * @param string  $gid system GUID
     * @param integer $use used status
     */
    public function use_system($gid, $use)
    {
        $data["in_use"] = intval($use);
        $this->ci->db->where('gid', $gid);
        $this->ci->db->update(PAYMENTS_SYSTEMS_TABLE, $data);
        unset($this->systems_cache[$gid]);
    }

    /**
     * Remove payment system object by identifier
     *
     * @param integer $id system identifier
     *
     * @return void
     */
    public function delete_system($id)
    {
        $this->ci->db->where("id", $id);
        $this->ci->db->delete(PAYMENTS_SYSTEMS_TABLE);

        return;
    }

    /**
     * Remove payment system object by GUID
     *
     * @param string $gid system GUID
     *
     * @return void
     */
    public function delete_system_by_gid($gid)
    {
        $this->ci->db->where("gid", $gid);
        $this->ci->db->delete(PAYMENTS_SYSTEMS_TABLE);
        unset($this->systems_cache[$gid]);

        return;
    }

    // drivers methods

    /**
     * Load payment system model
     *
     * @param string $system_gid system GUID
     *
     * @return boolean
     */
    public function load_driver($system_gid)
    {
        if ($this->current_driver_name == $system_gid) {
            return true;
        }
        $driver = ucfirst(strtolower($system_gid) . "_model");
        $driver_class = NS_MODULES. "payments\\models\\systems\\" . $driver;
        if (class_exists($driver_class)) {
            $this->driver = new $driver_class();
            $this->current_driver_name = $system_gid;
        } else {
            $this->current_driver_name = "";
            $this->driver = "";

            return false;
        }
    }

    /**
     * Validate settings of payment system object for saving to data source
     *
     * @param array $data settings data
     *
     * @return array
     */
    public function validate_system_settings($data)
    {
        if (empty($this->current_driver_name)) {
            return false;
        }

        return $this->driver->validate_settings($data);
    }

    /**
     * Validate operator object of payment system for saving to data source
     *
     * @param string $system_gid   system guid
     * @param string $operator_gid operator guid
     * @param array  $data         operator data
     *
     * @return array
     */
    public function validate_system_operator($system_gid, $operator_gid, $data)
    {
        $return = array('errors' => array(), 'data' => array());

        if (empty($operator_gid)) {
            $reference = $this->ci->pg_language->ds->get_reference($this->module_gid, $system_gid . '_operators');
            if (!empty($reference["option"])) {
                $array_keys = array_keys($reference["option"]);
            } else {
                $array_keys = array(0);
            }
            $return['data']['gid'] = strval(max($array_keys) + 1);
        } else {
            $return['data']['gid'] = strval($operator_gid);
        }

        $default_lang_id = $this->ci->pg_language->current_lang_id;
        if (isset($data['name_' . $default_lang_id])) {
            $return['data']['name'][$default_lang_id] = trim(strip_tags($data['name_' . $default_lang_id]));
            if (empty($return['data']['name'][$default_lang_id])) {
                $return['errors'][] = l('error_empty_system_operator', 'payments');
            } else {
                foreach ($this->ci->pg_language->languages as $lid => $lang_data) {
                    if ($lid == $default_lang_id) {
                        continue;
                    }
                    if (!isset($data['name_' . $lid]) || empty($data['name_' . $lid])) {
                        $return['data']['name'][$lid] = $return['data']['name'][$default_lang_id];
                    } else {
                        $return['data']['name'][$lid] = trim(strip_tags($data['name_' . $lid]));
                        if (empty($return['data']['name'][$lid])) {
                            $return['errors'][] = l('error_empty_system_operator', 'payments');
                            break;
                        }
                    }
                }
            }
        }

        return $return;
    }

    /**
     * Return properties data of payment system object
     *
     * @return array/false
     */
    public function get_system_data_map()
    {
        if (empty($this->current_driver_name)) {
            return false;
        }

        return $this->driver->get_settings_map();
    }

    /**
     * Return html data of payment system object
     *
     * @return array/false
     */
    public function get_html_data_map()
    {
        if (empty($this->current_driver_name)) {
            return false;
        }

        return $this->driver->get_html_map();
    }

    /**
     *  Load driver
     *
     *  @return void
     */
    public function get_system_data()
    {
        return $this->driver->payment_data;
    }

    /**
     * Send request to payment system
     *
     * @param string $system_gid   system GUID
     * @param array  $payment_data payment data
     *
     * @return void/false
     */
    public function action_request($system_gid, $payment_data)
    {
        if (empty($this->current_driver_name) || $this->current_driver_name != $system_gid) {
            $this->load_driver($system_gid);
        }
        if (!empty($this->current_driver_name)) {
            $system_settings = $this->get_system_by_gid($this->current_driver_name);
        } else {
            $system_settings = array();
        }
        if ($this->log_data) {
            $this->log_data($system_gid, "request", $payment_data);
        }
        if (method_exists($this->driver, 'func_request')) {
            return $this->driver->func_request($payment_data, $system_settings);
        } else {
            return false;
        }
    }

    /**
     * Process responce from payment system
     *
     * @param string $system_gid   system GUID
     * @param array  $payment_data payment data
     *
     * @return array
     */
    public function action_responce($system_gid, $payment_data)
    {
        if (empty($this->current_driver_name) || $this->current_driver_name != $system_gid) {
            $this->load_driver($system_gid);
        }
        if (!empty($this->current_driver_name)) {
            $system_settings = $this->get_system_by_gid($this->current_driver_name);
        } else {
            $system_settings = array();
        }
        if ($this->log_data) {
            $this->log_data($system_gid, "responce", $payment_data);
        }

        return $this->driver->func_responce($payment_data, $system_settings);
    }

    /**
     * Use html form
     *
     * @param string $system_gid system GUID
     *
     * @return boolean
     */
    public function action_html($system_gid)
    {
        if (empty($this->current_driver_name) || $this->current_driver_name != $system_gid) {
            $this->load_driver($system_gid);
        }
        if (method_exists($this->driver, 'func_html')) {
            return $this->driver->func_html();
        } else {
            return false;
        }
    }

    /**
     * Validate payment data for sending to payment system
     *
     * @param string $system_gid   system GUID
     * @param array  $payment_data payment data
     *
     * @return array
     */
    public function action_validate($system_gid, $payment_data)
    {
        if (empty($this->current_driver_name) || $this->current_driver_name != $system_gid) {
            $this->load_driver($system_gid);
        }
        if (!empty($this->current_driver_name)) {
            $system_settings = $this->get_system_by_gid($this->current_driver_name);
        } else {
            $system_settings = array();
        }

        return $this->driver->func_validate($payment_data, $system_settings);
    }

    /**
     * Logging payment
     *
     * @param string $system_gid system GUID
     * @param string $log_type   action type
     * @param array  $log_data   logging data
     *
     * @return void
     */
    public function log_data($system_gid, $log_type = "request", $log_data = array())
    {
        $data["system_gid"] = $system_gid;
        $data["date_add"] = date("Y-m-d H:i:s");
        $data["log_type"] = $log_type;
        $data["payment_data"] = serialize($log_data);
        $this->ci->db->insert(PAYMENTS_LOG_TABLE, $data);

        return;
    }

    /**
     * Add information data field of payment system by language
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function lang_dedicate_module_callback_add($lang_id = false)
    {
        if (!$lang_id) {
            return;
        }

        $this->ci->load->dbforge();

        $fields['info_data_' . $lang_id] = array('type' => 'TEXT', 'null' => true);
        $this->ci->dbforge->add_column(PAYMENTS_SYSTEMS_TABLE, $fields);

        $default_lang_id = $this->ci->pg_language->get_default_lang_id();
        if ($lang_id != $default_lang_id) {
            $this->ci->db->set('info_data_' . $lang_id, 'info_data_' . $default_lang_id, false);
            $this->ci->db->update(PAYMENTS_SYSTEMS_TABLE);
        }
    }

    /**
     * Remove information data field of payment system by language
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function lang_dedicate_module_callback_delete($lang_id = false)
    {
        if (!$lang_id) {
            return;
        }

        $this->ci->load->dbforge();

        $fields_exists = $this->ci->db->list_fields(PAYMENTS_SYSTEMS_TABLE);

        $fields = array('info_data_' . $lang_id);
        foreach ($fields as $field_name) {
            if (!in_array($field_name, $fields_exists)) {
                continue;
            }
            $this->ci->dbforge->drop_column(PAYMENTS_SYSTEMS_TABLE, $field_name);
        }
    }

    /**
     * Use javascript popup
     *
     * @param string $system_gid system GUID
     *
     * @return boolean
     */
    public function action_js($system_gid)
    {
        if (empty($this->current_driver_name) || $this->current_driver_name != $system_gid) {
            $this->load_driver($system_gid);
        }
        if (method_exists($this->driver, 'func_js')) {
            return $this->driver->func_js();
        } else {
            return false;
        }
    }

    /**
     * Return javascript code
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return string
     */
    public function get_js($payment_data, $system_settings)
    {
        if (empty($this->current_driver_name)) {
            return false;
        }

        return $this->driver->get_js($payment_data, $system_settings);
    }

    /**
     * Return code of disabled sections
     *
     * @param array $data payment sections
     *
     * @return integer
     */
    public function get_disable_code($data)
    {
        $data = array_intersect_key($this->payment_sections, array_flip((array) $data));
        if (empty($data)) {
            return 0;
        }
        $binary_string = "";
        $max = max($data);
        for ($i = 0; $i <= $max; ++$i) {
            $binary_string = ((in_array($i, $data)) ? "1" : "0") . $binary_string;
        }

        return bindec($binary_string);
    }

    /**
     * Save operator data to data source
     *
     * @param string $system_gid   system guid
     * @param string $operator_gid operator guid
     * @param array  $data         operator data
     *
     * @return array
     */
    public function save_system_operator($system_gid, $operator_gid, $data)
    {
        foreach ($data as $lid => $string) {
            $reference = $this->ci->pg_language->ds->get_reference($this->module_gid, $system_gid . '_operators', $lid);
            $reference["option"][$operator_gid] = $string;
            $this->ci->pg_language->ds->set_module_reference($this->module_gid, $system_gid . '_operators', $reference, $lid);
        }
    }

    /**
     * Save operator data to data source
     *
     * @param string $system_gid   system guid
     * @param string $operator_gid operator guid
     *
     * @return array
     */
    public function delete_system_operator($system_gid, $operator_gid)
    {
        $system_data = $this->get_system_by_gid($system_gid);
        if ($system_data['tarifs_status'][$operator_gid]) {
            unset($system_data['tarifs_status'][$operator_gid]);
        }
        if ($system_data['tarifs_data'][$operator_gid]) {
            unset($system_data['tarifs_data'][$operator_gid]);
        }
        $save_data = array('gid' => $system_data['gid'], 'tarifs_status' => $system_data['tarifs_status'], 'tarifs_data' => $system_data['tarifs_data']);
        $validate_data = $this->validate_system($system_data['id'], $save_data);
        $this->save_system($system_data['id'], $validate_data['data']);

        foreach ($this->ci->pg_language->languages as $lid => $lang_data) {
            $reference = $this->ci->pg_language->ds->get_reference($this->module_gid, $system_gid . '_operators', $lid);
            if (isset($reference["option"][$operator_gid])) {
                unset($reference["option"][$operator_gid]);
            }
            $this->ci->pg_language->ds->set_module_reference($this->module_gid, $system_gid . '_operators', $reference, $lid);
        }
    }

    /**
     * Validate payment system tarifs for saving to data source
     *
     * @param array $data tarifs data
     *
     * @return array
     */
    public function validate_tarifs_data($data)
    {
        $return = array('errors' => array(), 'data' => array());

        foreach ($data as $key => $values) {
            if (!is_array($values)) {
                continue;
            }
            foreach ($values as $k => $v) {
                $return['data'][$key][$k] = floatval($v);
                if (empty($return['data'][$key][$k])) {
                    unset($return['data'][$key][$k]);
                }
            }
        }

        return $return;
    }

    /**
     * Change sorting order of payment system operators
     *
     * @param string $system_gid  system guid
     * @param array  $sorter_data sorter data
     *
     * @return void
     */
    public function save_system_operators_sorter($system_gid, $sorter_data)
    {
        $this->ci->pg_language->ds->set_reference_sorter($this->module_gid, $system_gid . '_operators', $sorter_data);
    }

    /**
     * Return payment system rights
     *
     * @param string $system_data system data
     *
     * @return array
     */
    public function get_system_access($system_data)
    {
        $data = array();
        $payment_systems = array_flip($this->payment_sections);
        foreach ($payment_systems as $access) {
            $data[$access] = 1;
        }
        if ($system_data['disable']) {
            $binary_string = decbin($system_data['disable']);
            $arr = str_split($binary_string);
            $max = count($arr) - 1;
            for ($i = 0; $i <= $max; ++$i) {
                if ($arr[$max - $i] == 1) {
                    $data[$payment_systems[$i]] = 0;
                }
            }
        }

        return $data;
    }

}
